# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for clouds.
"""

import re
from gns3.qt import QtCore, QtWidgets
from ..ui.cloud_configuration_page_ui import Ui_cloudConfigPageWidget


class CloudConfigurationPage(QtWidgets.QWidget, Ui_cloudConfigPageWidget):

    """
    QWidget configuration page for clouds.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._nios = set()

        # connect NIO generic Ethernet slots
        self.uiGenericEthernetComboBox.currentIndexChanged.connect(self._genericEthernetSelectedSlot)
        self.uiGenericEthernetListWidget.itemSelectionChanged.connect(self._genericEthernetChangedSlot)
        self.uiAddGenericEthernetPushButton.clicked.connect(self._genericEthernetAddSlot)
        self.uiDeleteGenericEthernetPushButton.clicked.connect(self._genericEthernetDeleteSlot)

        # connect NIO Linux Ethernet slots
        self.uiLinuxEthernetComboBox.currentIndexChanged.connect(self._linuxEthernetSelectedSlot)
        self.uiLinuxEthernetListWidget.itemSelectionChanged.connect(self._linuxEthernetChangedSlot)
        self.uiAddLinuxEthernetPushButton.clicked.connect(self._linuxEthernetAddSlot)
        self.uiDeleteLinuxEthernetPushButton.clicked.connect(self._linuxEthernetDeleteSlot)

        # connect NIO NAT slots
        self.uiNIONATListWidget.currentRowChanged.connect(self._NIONATSelectedSlot)
        self.uiNIONATListWidget.itemSelectionChanged.connect(self._NIONATChangedSlot)
        self.uiAddNIONATPushButton.clicked.connect(self._NIONATAddSlot)
        self.uiDeleteNIONATPushButton.clicked.connect(self._NIONATDeleteSlot)

        # connect NIO UDP slots
        self.uiNIOUDPListWidget.currentRowChanged.connect(self._NIOUDPSelectedSlot)
        self.uiNIOUDPListWidget.itemSelectionChanged.connect(self._NIOUDPChangedSlot)
        self.uiAddNIOUDPPushButton.clicked.connect(self._NIOUDPAddSlot)
        self.uiDeleteNIOUDPPushButton.clicked.connect(self._NIOUDPDeleteSlot)

        # connect NIO TAP slots
        self.uiNIOTAPListWidget.currentRowChanged.connect(self._NIOTAPSelectedSlot)
        self.uiNIOTAPListWidget.itemSelectionChanged.connect(self._NIOTAPChangedSlot)
        self.uiAddNIOTAPPushButton.clicked.connect(self._NIOTAPAddSlot)
        self.uiDeleteNIOTAPPushButton.clicked.connect(self._NIOTAPDeleteSlot)

        # connect NIO UNIX slots
        self.uiNIOUNIXListWidget.currentRowChanged.connect(self._NIOUNIXSelectedSlot)
        self.uiNIOUNIXListWidget.itemSelectionChanged.connect(self._NIOUNIXChangedSlot)
        self.uiAddNIOUNIXPushButton.clicked.connect(self._NIOUNIXAddSlot)
        self.uiDeleteNIOUNIXPushButton.clicked.connect(self._NIOUNIXDeleteSlot)

        # connect NIO VDE slots
        self.uiNIOVDEListWidget.currentRowChanged.connect(self._NIOVDESelectedSlot)
        self.uiNIOVDEListWidget.itemSelectionChanged.connect(self._NIOVDEChangedSlot)
        self.uiAddNIOVDEPushButton.clicked.connect(self._NIOVDEAddSlot)
        self.uiDeleteNIOVDEPushButton.clicked.connect(self._NIOVDEDeleteSlot)

        # connect NIO NULL slots
        self.uiNIONullListWidget.currentRowChanged.connect(self._NIONullSelectedSlot)
        self.uiNIONullListWidget.itemSelectionChanged.connect(self._NIONullChangedSlot)
        self.uiAddNIONullPushButton.clicked.connect(self._NIONullAddSlot)
        self.uiDeleteNIONullPushButton.clicked.connect(self._NIONullDeleteSlot)

    def _genericEthernetSelectedSlot(self, index):
        """
        Loads the selected generic Ethernet interface in lineEdit.

        :param index: ignored
        """

        self.uiGenericEthernetLineEdit.setText(self.uiGenericEthernetComboBox.currentText())

    def _genericEthernetChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiGenericEthernetListWidget.currentItem()
        if item:
            self.uiDeleteGenericEthernetPushButton.setEnabled(True)
        else:
            self.uiDeleteGenericEthernetPushButton.setEnabled(False)

    def _genericEthernetAddSlot(self):
        """
        Adds a new generic Ethernet NIO.
        """

        interface = self.uiGenericEthernetLineEdit.text()
        if interface:
            nio = "nio_gen_eth:{interface}".format(interface=interface)
            if nio not in self._nios:
                self.uiGenericEthernetListWidget.addItem(nio)
                self._nios.add(nio)

    def _genericEthernetDeleteSlot(self):
        """
        Deletes the selected generic Ethernet NIO.
        """

        item = self.uiGenericEthernetListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiGenericEthernetListWidget.takeItem(self.uiGenericEthernetListWidget.currentRow())

    def _linuxEthernetSelectedSlot(self, index):
        """
        Loads the selected Linux interface in lineEdit.

        :param index: ignored
        """

        self.uiLinuxEthernetLineEdit.setText(self.uiLinuxEthernetComboBox.currentText())

    def _linuxEthernetChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiLinuxEthernetListWidget.currentItem()
        if item:
            self.uiDeleteLinuxEthernetPushButton.setEnabled(True)
        else:
            self.uiDeleteLinuxEthernetPushButton.setEnabled(False)

    def _linuxEthernetAddSlot(self):
        """
        Adds a new Linux Ethernet NIO.
        """

        interface = self.uiLinuxEthernetLineEdit.text()
        if interface:
            nio = "nio_gen_linux:{interface}".format(interface=interface)
            if nio not in self._nios:
                self.uiLinuxEthernetListWidget.addItem(nio)
                self._nios.add(nio)

    def _linuxEthernetDeleteSlot(self):
        """
        Deletes the selected Linux Ethernet NIO.
        """

        item = self.uiLinuxEthernetListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiLinuxEthernetListWidget.takeItem(self.uiLinuxEthernetListWidget.currentRow())

    def _NIONATSelectedSlot(self, index):
        """
        Loads a selected NAT NIO.

        :param index: ignored
        """

        item = self.uiNIONATListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_nat:(.+)$""", nio)
            if match:
                self.uiNIONATIdentiferLineEdit.setText(match.group(1))

    def _NIONATChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIONATListWidget.currentItem()
        if item:
            self.uiDeleteNIONATPushButton.setEnabled(True)
        else:
            self.uiDeleteNIONATPushButton.setEnabled(False)

    def _NIONATAddSlot(self):
        """
        Adds a new NAT NIO.
        """

        identifier = self.uiNIONATIdentiferLineEdit.text()
        if identifier:
            nio = "nio_nat:{}".format(identifier)
            if nio not in self._nios:
                self.uiNIONATListWidget.addItem(nio)
                self._nios.add(nio)

    def _NIONATDeleteSlot(self):
        """
        Deletes a NAT NIO.
        """

        item = self.uiNIONATListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIONATListWidget.takeItem(self.uiNIONATListWidget.currentRow())

    def _NIOUDPSelectedSlot(self, index):
        """
        Loads a selected UDP.

        :param index: ignored
        """

        item = self.uiNIOUDPListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_udp:(\d+):(.+):(\d+)$""", nio)
            if match:
                self.uiLocalPortSpinBox.setValue(int(match.group(1)))
                self.uiRemoteHostLineEdit.setText(match.group(2))
                self.uiRemotePortSpinBox.setValue(int(match.group(3)))

    def _NIOUDPChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIOUDPListWidget.currentItem()
        if item:
            self.uiDeleteNIOUDPPushButton.setEnabled(True)
        else:
            self.uiDeleteNIOUDPPushButton.setEnabled(False)

    def _NIOUDPAddSlot(self):
        """
        Adds a new UDP NIO.
        """

        local_port = self.uiLocalPortSpinBox.value()
        remote_host = self.uiRemoteHostLineEdit.text()
        remote_port = self.uiRemotePortSpinBox.value()
        if remote_host:
            nio = "nio_udp:{lport}:{rhost}:{rport}".format(lport=local_port,
                                                           rhost=remote_host,
                                                           rport=remote_port)
            if nio not in self._nios:
                self.uiNIOUDPListWidget.addItem(nio)
                self._nios.add(nio)
                self.uiLocalPortSpinBox.setValue(local_port + 1)
                self.uiRemotePortSpinBox.setValue(remote_port + 1)

    def _NIOUDPDeleteSlot(self):
        """
        Deletes an UDP NIO.
        """

        item = self.uiNIOUDPListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIOUDPListWidget.takeItem(self.uiNIOUDPListWidget.currentRow())

    def _NIOTAPSelectedSlot(self, index):
        """
        Loads the selected NIO TAP in lineEdit.

        :param index: ignored
        """

        item = self.uiNIOTAPListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_tap:(.+)$""", nio)
            if match:
                self.uiNIOTAPLineEdit.setText(match.group(1))

    def _NIOTAPChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIOTAPListWidget.currentItem()
        if item:
            self.uiDeleteNIOTAPPushButton.setEnabled(True)
        else:
            self.uiDeleteNIOTAPPushButton.setEnabled(False)

    def _NIOTAPAddSlot(self):
        """
        Adds a new UDP NIO.
        """

        tap_interface = self.uiNIOTAPLineEdit.text()
        if tap_interface:
            nio = "nio_tap:{}".format(tap_interface.lower())
            if nio not in self._nios:
                self.uiNIOTAPListWidget.addItem(nio)
                self._nios.add(nio)

    def _NIOTAPDeleteSlot(self):
        """
        Deletes a TAP NIO.
        """

        item = self.uiNIOTAPListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIOTAPListWidget.takeItem(self.uiNIOTAPListWidget.currentRow())

    def _NIOUNIXSelectedSlot(self, index):
        """
        Loads a selected UNIX NIO.

        :param index: ignored
        """

        item = self.uiNIOUNIXListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_unix:(.+):(.+)$""", nio)
            if match:
                self.uiLocalFileLineEdit.setText(match.group(1))
                self.uiRemoteFileLineEdit.setText(match.group(2))

    def _NIOUNIXChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIOUNIXListWidget.currentItem()
        if item:
            self.uiDeleteNIOUNIXPushButton.setEnabled(True)
        else:
            self.uiDeleteNIOUNIXPushButton.setEnabled(False)

    def _NIOUNIXAddSlot(self):
        """
        Adds a new UNIX NIO.
        """

        local_file = self.uiLocalFileLineEdit.text()
        remote_file = self.uiRemoteFileLineEdit.text()
        if local_file and remote_file:
            nio = "nio_unix:{local}:{remote}".format(local=local_file,
                                                     remote=remote_file)
            if nio not in self._nios:
                self.uiNIOUNIXListWidget.addItem(nio)
                self._nios.add(nio)

    def _NIOUNIXDeleteSlot(self):
        """
        Deletes an UNIX NIO.
        """

        item = self.uiNIOUNIXListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIOUNIXListWidget.takeItem(self.uiNIOUNIXListWidget.currentRow())

    def _NIOVDESelectedSlot(self, index):
        """
        Loads a selected VDE NIO.

        :param index: ignored
        """

        item = self.uiNIOVDEListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_vde:(.+):(.+)$""", nio)
            if match:
                self.uiVDEControlFileLineEdit.setText(match.group(1))
                self.uiVDELocalFileLineEdit.setText(match.group(2))

    def _NIOVDEChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIOVDEListWidget.currentItem()
        if item:
            self.uiDeleteNIOVDEPushButton.setEnabled(True)
        else:
            self.uiDeleteNIOVDEPushButton.setEnabled(False)

    def _NIOVDEAddSlot(self):
        """
        Adds a new VDE NIO.
        """

        control_file = self.uiVDEControlFileLineEdit.text()
        local_file = self.uiVDELocalFileLineEdit.text()
        if local_file and control_file:
            nio = "nio_vde:{control}:{local}".format(control=control_file, local=local_file)
            if nio not in self._nios:
                self.uiNIOVDEListWidget.addItem(nio)
                self._nios.add(nio)

    def _NIOVDEDeleteSlot(self):
        """
        Deletes a VDE NIO.
        """

        item = self.uiNIOVDEListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIOVDEListWidget.takeItem(self.uiNIOVDEListWidget.currentRow())

    def _NIONullSelectedSlot(self, index):
        """
        Loads a selected NULL NIO.

        :param index: ignored
        """

        item = self.uiNIONullListWidget.currentItem()
        if item:
            nio = item.text()
            match = re.search(r"""^nio_null:(.+)$""", nio)
            if match:
                self.uiNIONullIdentiferLineEdit.setText(match.group(1))

    def _NIONullChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiNIONullListWidget.currentItem()
        if item:
            self.uiDeleteNIONullPushButton.setEnabled(True)
        else:
            self.uiDeleteNIONullPushButton.setEnabled(False)

    def _NIONullAddSlot(self):
        """
        Adds a new NULL NIO.
        """

        identifier = self.uiNIONullIdentiferLineEdit.text()
        if identifier:
            nio = "nio_null:{}".format(identifier)
            if nio not in self._nios:
                self.uiNIONullListWidget.addItem(nio)
                self._nios.add(nio)

    def _NIONullDeleteSlot(self):
        """
        Deletes a NULL NIO.
        """

        item = self.uiNIONullListWidget.currentItem()
        if item:
            nio = item.text()
            # check we can delete that NIO
            node_ports = self._node.ports()
            for node_port in node_ports:
                if node_port.name() == nio and not node_port.isFree():
                    QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to NIO {}, please remove it first".format(nio))
                    return
            self._nios.remove(nio)
            self.uiNIONullListWidget.takeItem(self.uiNIONullListWidget.currentRow())

    def loadSettings(self, settings, node, group=False):
        """
        Loads the cloud settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
        else:
            self.uiNameLineEdit.setEnabled(False)

        self._node = node

        # load all network interfaces
        self.uiGenericEthernetComboBox.clear()
        index = 0
        for interface in settings["interfaces"]:
            if interface["name"].startswith("tap"):
                # do not add TAP interfaces
                continue
            self.uiGenericEthernetComboBox.addItem(interface["name"])
            self.uiGenericEthernetComboBox.setItemData(index, interface["id"], QtCore.Qt.ToolTipRole)
            index += 1
        self.uiGenericEthernetComboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)

        # load all network interfaces
        self.uiLinuxEthernetComboBox.clear()
        index = 0
        for interface in settings["interfaces"]:
            if not interface["name"].startswith(r"\Device\NPF_") and not interface["name"].startswith("tap"):
                self.uiLinuxEthernetComboBox.addItem(interface["name"])
                self.uiLinuxEthernetComboBox.setItemData(index, interface["id"], QtCore.Qt.ToolTipRole)
                index += 1
        self.uiLinuxEthernetComboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)

        # populate the NIO lists
        self.nios = set()
        self.uiGenericEthernetListWidget.clear()
        self.uiLinuxEthernetListWidget.clear()
        self.uiNIOUDPListWidget.clear()
        self.uiNIOTAPListWidget.clear()
        self.uiNIOUNIXListWidget.clear()
        self.uiNIOVDEListWidget.clear()
        self.uiNIONullListWidget.clear()

        for nio in settings["nios"]:
            self._nios.add(nio)
            if nio.lower().startswith("nio_gen_eth"):
                self.uiGenericEthernetListWidget.addItem(nio)
            elif nio.lower().startswith("nio_gen_linux"):
                self.uiLinuxEthernetListWidget.addItem(nio)
            elif nio.lower().startswith("nio_udp"):
                self.uiNIOUDPListWidget.addItem(nio)
            elif nio.lower().startswith("nio_tap"):
                self.uiNIOTAPListWidget.addItem(nio)
            elif nio.lower().startswith("nio_unix"):
                self.uiNIOUNIXListWidget.addItem(nio)
            elif nio.lower().startswith("nio_vde"):
                self.uiNIOVDEListWidget.addItem(nio)
            elif nio.lower().startswith("nio_null"):
                self.uiNIONullListWidget.addItem(nio)

    def saveSettings(self, settings, node, group=False):
        """
        Saves the cloud settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            settings["name"] = self.uiNameLineEdit.text()
        else:
            del settings["name"]

        settings["nios"] = list(self._nios)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Base class for shape items (Rectangle, ellipse etc.).
"""

from ..qt import QtCore, QtGui, QtWidgets


class ShapeItem:

    """
    Base class to draw shapes on the scene.
    """

    show_layer = False

    def __init__(self, **kws):

        self.setFlags(QtWidgets.QGraphicsItem.ItemIsMovable | QtWidgets.QGraphicsItem.ItemIsFocusable | QtWidgets.QGraphicsItem.ItemIsSelectable)
        self.setAcceptHoverEvents(True)
        self._border = 5
        self._edge = None

        from ..main_window import MainWindow
        self._graphics_view = MainWindow.instance().uiGraphicsView

    def keyPressEvent(self, event):
        """
        Handles all key press events

        :param event: QKeyEvent
        """

        key = event.key()
        modifiers = event.modifiers()
        if key in (QtCore.Qt.Key_P, QtCore.Qt.Key_Plus, QtCore.Qt.Key_Equal) and modifiers & QtCore.Qt.AltModifier \
                or key == QtCore.Qt.Key_Plus and modifiers & QtCore.Qt.AltModifier and modifiers & QtCore.Qt.KeypadModifier:
            if self.rotation() > -360.0:
                self.setRotation(self.rotation() - 1)
        elif key in (QtCore.Qt.Key_M, QtCore.Qt.Key_Minus) and modifiers & QtCore.Qt.AltModifier \
                or key == QtCore.Qt.Key_Minus and modifiers & QtCore.Qt.AltModifier and modifiers & QtCore.Qt.KeypadModifier:
            if self.rotation() < 360.0:
                self.setRotation(self.rotation() + 1)
        else:
            QtWidgets.QGraphicsItem.keyPressEvent(self, event)

    def mousePressEvent(self, event):
        """
        Handles all mouse press events.

        :param event: QMouseEvent instance
        """

        self.update()
        if event.pos().x() > (self.rect().right() - self._border):
            self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, False)
            self._edge = "right"

        elif event.pos().x() < (self.rect().left() + self._border):
            self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, False)
            self._edge = "left"

        elif event.pos().y() < (self.rect().top() + self._border):
            self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, False)
            self._edge = "top"

        elif event.pos().y() > (self.rect().bottom() - self._border):
            self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable, False)
            self._edge = "bottom"

        QtWidgets.QGraphicsItem.mousePressEvent(self, event)

    def mouseReleaseEvent(self, event):
        """
        Handles all mouse release events.

        :param: QMouseEvent instance
        """

        self.update()
        self.setFlag(QtWidgets.QGraphicsItem.ItemIsMovable)
        self._edge = None
        QtWidgets.QGraphicsItem.mouseReleaseEvent(self, event)

    def mouseMoveEvent(self, event):
        """
        Handles all mouse move events.

        :param event: QMouseEvent instance
        """

        self.update()
        if self._edge:
            r = self.rect()
            scenePos = event.scenePos()

            if self._edge == "top":
                diff = self.y() - scenePos.y()
                if r.height() - diff > 0:
                    self.setPos(self.x(), scenePos.y())
                    self.setRect(0, 0, self.rect().width(), self.rect().height() + diff)
                else:
                    self._edge = "bottom"
                    self.setPos(self.x(), self.y() + self.rect().height())
                    self.setRect(0, 0, self.rect().width(), diff - self.rect().height())
            elif self._edge == "left":
                diff = self.x() - scenePos.x()
                if r.width() - diff > 0:
                    self.setPos(scenePos.x(), self.y())
                    self.setRect(0, 0, r.width() + diff, self.rect().height())
                else:
                    self._edge = "right"
                    self.setPos(self.x() + self.rect().width(), self.y())
                    self.setRect(0, 0, diff - self.rect().width(), self.rect().height())
            elif self._edge == "bottom":
                if r.height() > 0:
                    pos = self.mapFromScene(scenePos)
                    self.setRect(0, 0, self.rect().width(), pos.y())
                else:
                    self.setRect(0, 0, self.rect().width(), abs(scenePos.y() - self.y()))
                    self.setPos(self.x(), scenePos.y())
                    self._edge = "top"
            elif self._edge == "right":
                if r.width() > 0:
                    pos = self.mapFromScene(scenePos)
                    self.setRect(0, 0, pos.x(), self.rect().height())
                else:
                    self.setRect(0, 0, abs(scenePos.x() - self.x()), self.rect().height())
                    self.setPos(scenePos.x(), self.y())
                    self._edge = "left"

        QtWidgets.QGraphicsItem.mouseMoveEvent(self, event)

    def hoverMoveEvent(self, event):
        """
        Handles all hover move events.

        :param event: QGraphicsSceneHoverEvent instance
        """

        # objects on the background layer don't need cursors
        if self.zValue() >= 0:
            if event.pos().x() > (self.rect().right() - self._border):
                self._graphics_view.setCursor(QtCore.Qt.SizeHorCursor)
            elif event.pos().x() < (self.rect().left() + self._border):
                self._graphics_view.setCursor(QtCore.Qt.SizeHorCursor)
            elif event.pos().y() < (self.rect().top() + self._border):
                self._graphics_view.setCursor(QtCore.Qt.SizeVerCursor)
            elif event.pos().y() > (self.rect().bottom() - self._border):
                self._graphics_view.setCursor(QtCore.Qt.SizeVerCursor)
            else:
                self._graphics_view.setCursor(QtCore.Qt.SizeAllCursor)

    def hoverLeaveEvent(self, event):
        """
        Handles all hover leave events.

        :param event: QGraphicsSceneHoverEvent instance
        """

        # objects on the background layer don't need cursors
        if self.zValue() >= 0:
            self._graphics_view.setCursor(QtCore.Qt.ArrowCursor)

    def drawLayerInfo(self, painter):
        """
        Draws the layer position.

        :param painter: QPainter instance
        """

        if self.show_layer is False:
            return

        brect = self.boundingRect()
        # don't draw anything if the object is too small
        if brect.width() < 20 or brect.height() < 20:
            return

        center = self.mapFromItem(self, brect.width() / 2.0, brect.height() / 2.0)
        painter.setBrush(QtCore.Qt.red)
        painter.setPen(QtCore.Qt.red)
        painter.drawRect((brect.width() / 2.0) - 10, (brect.height() / 2.0) - 10, 20, 20)
        painter.setPen(QtCore.Qt.black)
        zval = str(int(self.zValue()))
        painter.drawText(QtCore.QPointF(center.x() - 4, center.y() + 4), zval)

    def setZValue(self, value):
        """
        Sets a new Z value.

        :param value: Z value
        """

        QtWidgets.QGraphicsItem.setZValue(self, value)
        if self.zValue() < 0:
            self.setFlag(self.ItemIsSelectable, False)
            self.setFlag(self.ItemIsMovable, False)
        else:
            self.setFlag(self.ItemIsSelectable, True)
            self.setFlag(self.ItemIsMovable, True)

    def dump(self):
        """
        Returns a representation of this shape item.

        :returns: dictionary
        """

        shape_info = {"width": self.rect().width(),
                      "height": self.rect().height(),
                      "x": self.x(),
                      "y": self.y()}

        brush = self.brush()
        if brush.color() != QtCore.Qt.white:
            shape_info["color"] = brush.color().name()
        if brush.color().alpha() != 255:
            shape_info["transparency"] = brush.color().alpha()

        pen = self.pen()
        if pen.color() != QtCore.Qt.black:
            shape_info["border_color"] = pen.color().name()
        if pen.color().alpha() != 255:
            shape_info["border_transparency"] = pen.color().alpha()
        if pen.width() != 2:
            shape_info["border_width"] = pen.width()
        if pen.style() != QtCore.Qt.SolidLine:
            shape_info["border_style"] = pen.style()

        if self.rotation() != 0:
            shape_info["rotation"] = self.rotation()
        if self.zValue() != 0:
            shape_info["z"] = self.zValue()

        return shape_info

    def load(self, shape_info):
        """
        Loads a representation of this shape item.
        (from a topology file).

        :param shape_info: representation of the shape item (dictionary)
        """

        # load mandatory properties
        width = shape_info["width"]
        height = shape_info["height"]
        x = shape_info["x"]
        y = shape_info["y"]
        self.setRect(0, 0, width, height)
        self.setPos(x, y)

        # load optional properties
        z = shape_info.get("z")
        color = shape_info.get("color")
        if not color and shape_info.get("fill_color"):
            # compatibility with old 1.0 projects
            color = shape_info.get("fill_color")
        transparency = shape_info.get("transparency")
        border_color = shape_info.get("border_color")
        border_transparency = shape_info.get("border_transparency")
        border_width = shape_info.get("border_width")
        border_style = shape_info.get("border_style")
        rotation = shape_info.get("rotation")

        if color:
            color = QtGui.QColor(color)
        else:
            color = QtGui.QColor(255, 255, 255)
        if transparency is not None:
            color.setAlpha(transparency)
        self.setBrush(QtGui.QBrush(color))

        pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
        if border_color:
            border_color = QtGui.QColor(border_color)
        else:
            border_color = pen.color()
        if border_transparency:
            border_color.setAlpha(border_transparency)
        pen.setColor(border_color)
        if border_width is not None:
            pen.setWidth(int(border_width))
        if border_style is not None:
            pen.setStyle(QtCore.Qt.PenStyle(border_style))
        self.setPen(pen)

        if rotation is not None:
            self.setRotation(rotation)
        if z is not None:
            self.setZValue(z)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/setup_wizard.ui'
#
# Created: Thu May  5 16:41:20 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_SetupWizard(object):
    def setupUi(self, SetupWizard):
        SetupWizard.setObjectName("SetupWizard")
        SetupWizard.resize(775, 539)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SetupWizard.sizePolicy().hasHeightForWidth())
        SetupWizard.setSizePolicy(sizePolicy)
        SetupWizard.setMaximumSize(QtCore.QSize(775, 563))
        SetupWizard.setModal(True)
        SetupWizard.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        SetupWizard.setOptions(QtWidgets.QWizard.NoBackButtonOnStartPage)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiServerWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerWizardPage)
        self.uiVMRadioButton.setChecked(True)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.gridLayout.addWidget(self.uiVMRadioButton, 0, 0, 1, 1)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerWizardPage)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.gridLayout.addWidget(self.uiLocalRadioButton, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 180, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 0, 1, 1)
        self.uiShowCheckBox = QtWidgets.QCheckBox(self.uiServerWizardPage)
        self.uiShowCheckBox.setChecked(True)
        self.uiShowCheckBox.setObjectName("uiShowCheckBox")
        self.gridLayout.addWidget(self.uiShowCheckBox, 3, 0, 1, 1)
        SetupWizard.addPage(self.uiServerWizardPage)
        self.uiVMWizardPage = QtWidgets.QWizardPage()
        self.uiVMWizardPage.setObjectName("uiVMWizardPage")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiVMWizardPage)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiVirtualizationSoftwarLabel = QtWidgets.QLabel(self.uiVMWizardPage)
        self.uiVirtualizationSoftwarLabel.setObjectName("uiVirtualizationSoftwarLabel")
        self.verticalLayout.addWidget(self.uiVirtualizationSoftwarLabel)
        self.uiVmwareRadioButton = QtWidgets.QRadioButton(self.uiVMWizardPage)
        self.uiVmwareRadioButton.setChecked(True)
        self.uiVmwareRadioButton.setObjectName("uiVmwareRadioButton")
        self.verticalLayout.addWidget(self.uiVmwareRadioButton)
        self.uiVirtualBoxRadioButton = QtWidgets.QRadioButton(self.uiVMWizardPage)
        self.uiVirtualBoxRadioButton.setObjectName("uiVirtualBoxRadioButton")
        self.verticalLayout.addWidget(self.uiVirtualBoxRadioButton)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.horizontalLayout_2.addLayout(self.verticalLayout)
        self.uiVMwareBannerButton = QtWidgets.QPushButton(self.uiVMWizardPage)
        self.uiVMwareBannerButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/vmware_fusion_banner.jpg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.uiVMwareBannerButton.setIcon(icon)
        self.uiVMwareBannerButton.setIconSize(QtCore.QSize(454, 150))
        self.uiVMwareBannerButton.setFlat(True)
        self.uiVMwareBannerButton.setObjectName("uiVMwareBannerButton")
        self.horizontalLayout_2.addWidget(self.uiVMwareBannerButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.uiGNS3VMDownloadLinkUrlLabel = QtWidgets.QLabel(self.uiVMWizardPage)
        self.uiGNS3VMDownloadLinkUrlLabel.setOpenExternalLinks(True)
        self.uiGNS3VMDownloadLinkUrlLabel.setObjectName("uiGNS3VMDownloadLinkUrlLabel")
        self.verticalLayout_2.addWidget(self.uiGNS3VMDownloadLinkUrlLabel)
        self.uiVMNameLabel = QtWidgets.QLabel(self.uiVMWizardPage)
        self.uiVMNameLabel.setObjectName("uiVMNameLabel")
        self.verticalLayout_2.addWidget(self.uiVMNameLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiVMListComboBox = QtWidgets.QComboBox(self.uiVMWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMListComboBox.sizePolicy().hasHeightForWidth())
        self.uiVMListComboBox.setSizePolicy(sizePolicy)
        self.uiVMListComboBox.setObjectName("uiVMListComboBox")
        self.horizontalLayout.addWidget(self.uiVMListComboBox)
        self.uiRefreshPushButton = QtWidgets.QPushButton(self.uiVMWizardPage)
        self.uiRefreshPushButton.setObjectName("uiRefreshPushButton")
        self.horizontalLayout.addWidget(self.uiRefreshPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.uiCPULabel = QtWidgets.QLabel(self.uiVMWizardPage)
        self.uiCPULabel.setObjectName("uiCPULabel")
        self.verticalLayout_2.addWidget(self.uiCPULabel)
        self.uiCPUSpinBox = QtWidgets.QSpinBox(self.uiVMWizardPage)
        self.uiCPUSpinBox.setMinimum(1)
        self.uiCPUSpinBox.setMaximum(128)
        self.uiCPUSpinBox.setProperty("value", 1)
        self.uiCPUSpinBox.setObjectName("uiCPUSpinBox")
        self.verticalLayout_2.addWidget(self.uiCPUSpinBox)
        self.uiRAMLabel = QtWidgets.QLabel(self.uiVMWizardPage)
        self.uiRAMLabel.setObjectName("uiRAMLabel")
        self.verticalLayout_2.addWidget(self.uiRAMLabel)
        self.uiRAMSpinBox = QtWidgets.QSpinBox(self.uiVMWizardPage)
        self.uiRAMSpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRAMSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRAMSpinBox.setSizePolicy(sizePolicy)
        self.uiRAMSpinBox.setMinimum(512)
        self.uiRAMSpinBox.setMaximum(1000000000)
        self.uiRAMSpinBox.setSingleStep(1024)
        self.uiRAMSpinBox.setProperty("value", 2048)
        self.uiRAMSpinBox.setObjectName("uiRAMSpinBox")
        self.verticalLayout_2.addWidget(self.uiRAMSpinBox)
        SetupWizard.addPage(self.uiVMWizardPage)
        self.uiAddVMsWizardPage = QtWidgets.QWizardPage()
        self.uiAddVMsWizardPage.setObjectName("uiAddVMsWizardPage")
        self.uiAddIOSRouterCheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddIOSRouterCheckBox.setGeometry(QtCore.QRect(9, 9, 469, 22))
        self.uiAddIOSRouterCheckBox.setChecked(True)
        self.uiAddIOSRouterCheckBox.setObjectName("uiAddIOSRouterCheckBox")
        self.uiAddIOUDeviceCheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddIOUDeviceCheckBox.setGeometry(QtCore.QRect(9, 37, 426, 22))
        self.uiAddIOUDeviceCheckBox.setObjectName("uiAddIOUDeviceCheckBox")
        self.uiAddQemuVMcheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddQemuVMcheckBox.setGeometry(QtCore.QRect(9, 65, 219, 22))
        self.uiAddQemuVMcheckBox.setObjectName("uiAddQemuVMcheckBox")
        self.uiAddVirtualBoxVMcheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddVirtualBoxVMcheckBox.setGeometry(QtCore.QRect(9, 93, 249, 22))
        self.uiAddVirtualBoxVMcheckBox.setObjectName("uiAddVirtualBoxVMcheckBox")
        self.uiAddVMwareVMcheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddVMwareVMcheckBox.setGeometry(QtCore.QRect(9, 121, 233, 22))
        self.uiAddVMwareVMcheckBox.setObjectName("uiAddVMwareVMcheckBox")
        self.uiAddDockerVMCheckBox = QtWidgets.QCheckBox(self.uiAddVMsWizardPage)
        self.uiAddDockerVMCheckBox.setGeometry(QtCore.QRect(10, 150, 233, 22))
        self.uiAddDockerVMCheckBox.setObjectName("uiAddDockerVMCheckBox")
        SetupWizard.addPage(self.uiAddVMsWizardPage)

        self.retranslateUi(SetupWizard)
        QtCore.QMetaObject.connectSlotsByName(SetupWizard)

    def retranslateUi(self, SetupWizard):
        _translate = QtCore.QCoreApplication.translate
        SetupWizard.setWindowTitle(_translate("SetupWizard", "Setup Wizard"))
        self.uiServerWizardPage.setTitle(_translate("SetupWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("SetupWizard", "Please choose a server type to run your GNS3 network simulations. The GNS3 VM is strongly recommended on Windows and Mac OS X."))
        self.uiVMRadioButton.setToolTip(_translate("SetupWizard", "Dynamips, IOU, VPCS and Qemu will use this virtual machine."))
        self.uiVMRadioButton.setText(_translate("SetupWizard", "Local GNS3 VM"))
        self.uiLocalRadioButton.setToolTip(_translate("SetupWizard", "Eveything that is supported by your system will run on your computer."))
        self.uiLocalRadioButton.setText(_translate("SetupWizard", "Local server"))
        self.uiShowCheckBox.setText(_translate("SetupWizard", "Don\'t show this again"))
        self.uiVMWizardPage.setTitle(_translate("SetupWizard", "GNS3 VM"))
        self.uiVMWizardPage.setSubTitle(_translate("SetupWizard", "In order to run the GNS3 VM you must first have VMware or VirtualBox installed and the GNS3 VM.ova imported in one of these."))
        self.uiVirtualizationSoftwarLabel.setText(_translate("SetupWizard", "Virtualization software:"))
        self.uiVmwareRadioButton.setToolTip(_translate("SetupWizard", "VMware is recommended to run Qemu based appliances (required for KVM)."))
        self.uiVmwareRadioButton.setText(_translate("SetupWizard", "VMware (recommended)"))
        self.uiVirtualBoxRadioButton.setToolTip(_translate("SetupWizard", "Use VirtualBox if you intend to only use Dynamips, IOU or VPCS."))
        self.uiVirtualBoxRadioButton.setText(_translate("SetupWizard", "VirtualBox"))
        self.uiGNS3VMDownloadLinkUrlLabel.setText(_translate("SetupWizard", "<html><head/><body><p>The GNS3 VM can be <a href=\"https://github.com/GNS3/gns3-gui/releases/download/v1.4.1/GNS3.VM.VMware.Workstation.1.4.1.zip\"><span style=\" text-decoration: underline; color:#0000ff;\">downloaded here</span></a>. Import the VM in your virtualization software and hit refresh.</p></body></html>"))
        self.uiVMNameLabel.setText(_translate("SetupWizard", "VM name:"))
        self.uiRefreshPushButton.setText(_translate("SetupWizard", "&Refresh"))
        self.uiCPULabel.setText(_translate("SetupWizard", "vCPU cores:"))
        self.uiRAMLabel.setText(_translate("SetupWizard", "RAM size:"))
        self.uiRAMSpinBox.setSuffix(_translate("SetupWizard", " MB"))
        self.uiAddVMsWizardPage.setTitle(_translate("SetupWizard", "Add virtual machines"))
        self.uiAddVMsWizardPage.setSubTitle(_translate("SetupWizard", "Now that you have configured the server type you can choose to add one or more virtual machines (VMs) of different types."))
        self.uiAddIOSRouterCheckBox.setText(_translate("SetupWizard", "&Add an IOS router using a real IOS image (supported by Dynamips)"))
        self.uiAddIOUDeviceCheckBox.setText(_translate("SetupWizard", "&Add an IOU (IOS on UNIX) device using a L3 or L2 IOU image"))
        self.uiAddQemuVMcheckBox.setText(_translate("SetupWizard", "&Add a Qemu virtual machine"))
        self.uiAddVirtualBoxVMcheckBox.setText(_translate("SetupWizard", "&Add a VirtualBox virtual machine"))
        self.uiAddVMwareVMcheckBox.setText(_translate("SetupWizard", "&Add a VMware virtual machine"))
        self.uiAddDockerVMCheckBox.setText(_translate("SetupWizard", "&Add a Docker container"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from ..qt import QtCore, QtGui, QtWidgets
from ..ui.new_project_dialog_ui import Ui_NewProjectDialog


class NewProjectDialog(QtWidgets.QDialog, Ui_NewProjectDialog):

    """
    New project dialog.

    :param parent: parent widget.
    :param showed_from_startup: boolean to indicate if this dialog
    :param default_project_name: Project name by default
    has been opened automatically when GNS3 started.
    """

    def __init__(self, parent, showed_from_startup=False, default_project_name="untitled"):

        super().__init__(parent)
        self.setupUi(self)

        self._main_window = parent
        self._project_settings = {}
        self.uiNameLineEdit.setText(default_project_name)
        self.uiLocationLineEdit.setText(os.path.join(self._main_window.projectsDirPath(), default_project_name))

        self.uiNameLineEdit.textEdited.connect(self._projectNameSlot)
        self.uiLocationBrowserToolButton.clicked.connect(self._projectPathSlot)
        self.uiOpenProjectPushButton.clicked.connect(self._openProjectActionSlot)
        self.uiRecentProjectsPushButton.clicked.connect(self._showRecentProjectsSlot)

        if not showed_from_startup:
            self.uiOpenProjectPushButton.hide()
            self.uiRecentProjectsPushButton.hide()

    def keyPressEvent(self, e):
        """
        Event handler in order to properly handle escape.
        """

        if e.key() == QtCore.Qt.Key_Escape:
            self.close()

    def _projectNameSlot(self, text):

        project_dir = self._main_window.projectsDirPath()
        if os.path.dirname(self.uiLocationLineEdit.text()) == project_dir:
            self.uiLocationLineEdit.setText(os.path.join(project_dir, text))

    def _projectPathSlot(self):
        """
        Slot to select the a new project location.
        """

        path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Project location", os.path.join(self._main_window.projectsDirPath(),
                                                                                               self.uiNameLineEdit.text()))

        if path:
            self.uiLocationLineEdit.setText(path)

    def getNewProjectSettings(self):

        return self._project_settings

    def _menuTriggeredSlot(self, action):
        """
        Closes this dialog when a recent project
        has been opened.

        :param action: ignored.
        """

        self.reject()

    def _openProjectActionSlot(self):
        """
        Opens a project and closes this dialog.
        """

        self._main_window.openProjectActionSlot()
        self.reject()

    def _showRecentProjectsSlot(self):
        """
        lot to show all the recent projects in a menu.
        """

        menu = QtWidgets.QMenu()
        menu.triggered.connect(self._menuTriggeredSlot)
        for action in self._main_window._recent_file_actions:
            menu.addAction(action)
        menu.exec_(QtGui.QCursor.pos())

    def done(self, result):

        if result:
            project_name = self.uiNameLineEdit.text()
            project_location = self.uiLocationLineEdit.text()
            project_type = "local"

            if not project_name:
                QtWidgets.QMessageBox.critical(self, "New project", "Project name is empty")
                return

            if not project_location:
                QtWidgets.QMessageBox.critical(self, "New project", "Project location is empty")
                return

            if os.path.isdir(project_location):
                reply = QtWidgets.QMessageBox.question(self,
                                                       "New project",
                                                       "Location {} already exists, overwrite it?".format(project_location),
                                                       QtWidgets.QMessageBox.Yes,
                                                       QtWidgets.QMessageBox.No)
                if reply == QtWidgets.QMessageBox.No:
                    return

            self._project_settings["project_name"] = project_name
            self._project_settings["project_path"] = os.path.join(project_location, project_name + ".gns3")
            self._project_settings["project_files_dir"] = project_location
            self._project_settings["project_type"] = project_type

        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VirtualBox module implementation.
"""

import os
import sys
import shutil

from gns3.qt import QtWidgets
from gns3.local_server_config import LocalServerConfig
from gns3.local_config import LocalConfig

from ..module import Module
from ..module_error import ModuleError
from .virtualbox_vm import VirtualBoxVM
from .settings import VBOX_SETTINGS
from .settings import VBOX_VM_SETTINGS

import logging
log = logging.getLogger(__name__)


class VirtualBox(Module):

    """
    VirtualBox module.
    """

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._virtualbox_vms = {}
        self._nodes = []

        self.configChangedSlot()

    def configChangedSlot(self):
        # load the settings
        self._loadSettings()

    @staticmethod
    def _findVBoxManage(self):
        """
        Finds the VBoxManage path.

        :return: path to VBoxManage
        """

        vboxmanage_path = None
        if sys.platform.startswith("win"):
            if "VBOX_INSTALL_PATH" in os.environ:
                vboxmanage_path_windows = os.path.join(os.environ["VBOX_INSTALL_PATH"], "VBoxManage.exe")
                if os.path.exists(vboxmanage_path_windows):
                    vboxmanage_path = vboxmanage_path_windows
            elif "VBOX_MSI_INSTALL_PATH" in os.environ:
                vboxmanage_path_windows = os.path.join(os.environ["VBOX_MSI_INSTALL_PATH"], "VBoxManage.exe")
                if os.path.exists(vboxmanage_path_windows):
                    vboxmanage_path = vboxmanage_path_windows
        elif sys.platform.startswith("darwin"):
            vboxmanage_path_osx = "/Applications/VirtualBox.app/Contents/MacOS/VBoxManage"
            if os.path.exists(vboxmanage_path_osx):
                vboxmanage_path = vboxmanage_path_osx
        else:
            vboxmanage_path = shutil.which("vboxmanage")

        if vboxmanage_path is None:
            return ""
        return os.path.abspath(vboxmanage_path)

    def _loadSettings(self):
        """
        Loads the settings from the server settings file.
        """

        self._settings = LocalConfig.instance().loadSectionSettings(self.__class__.__name__, VBOX_SETTINGS)

        if not os.path.exists(self._settings["vboxmanage_path"]):
            self._settings["vboxmanage_path"] = self._findVBoxManage(self)

        self._loadVirtualBoxVMs()

    def _saveSettings(self):
        """
        Saves the settings to the server settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # save some settings to the local server config file
        if self._settings["vboxmanage_path"]:
            server_settings = {
                "vboxmanage_path": os.path.normpath(self._settings["vboxmanage_path"])
            }
            LocalServerConfig.instance().saveSettings(self.__class__.__name__, server_settings)

    def _loadVirtualBoxVMs(self):
        """
        Load the VirtualBox VMs from the client settings file.
        """

        settings = LocalConfig.instance().settings()
        if "vms" in settings.get(self.__class__.__name__, {}):
            for vm in settings[self.__class__.__name__]["vms"]:
                vmname = vm.get("vmname")
                server = vm.get("server")
                key = "{server}:{vmname}".format(server=server, vmname=vmname)
                if key in self._virtualbox_vms or not vmname or not server:
                    continue
                vm_settings = VBOX_VM_SETTINGS.copy()
                vm_settings.update(vm)
                # For backward compatibility we use vmname
                if not vm_settings["name"]:
                    vm_settings["name"] = vmname
                # for backward compatibility before version 1.4
                if "symbol" not in vm_settings:
                    vm_settings["symbol"] = vm_settings.get("default_symbol", vm_settings["symbol"])
                    vm_settings["symbol"] = vm_settings["symbol"][:-11] + ".svg" if vm_settings["symbol"].endswith("normal.svg") else vm_settings["symbol"]
                self._virtualbox_vms[key] = vm_settings

    def _saveVirtualBoxVMs(self):
        """
        Saves the VirtualBox VMs to the client settings file.
        """

        self._settings["vms"] = list(self._virtualbox_vms.values())
        self._saveSettings()

    def VMs(self):
        """
        Returns VirtualBox VMs settings.

        :returns: VirtualBox VMs settings (dictionary)
        """

        return self._virtualbox_vms

    @staticmethod
    def vmConfigurationPage():
        from .pages.virtualbox_vm_configuration_page import VirtualBoxVMConfigurationPage
        return VirtualBoxVMConfigurationPage

    def setVMs(self, new_virtualbox_vms):
        """
        Sets VirtualBox VM settings.

        :param new_virtualbox_vms: VirtualBox VM settings (dictionary)
        """

        self._virtualbox_vms = new_virtualbox_vms.copy()
        self._saveVirtualBoxVMs()

    def addNode(self, node):
        """
        Adds a node to this module.

        :param node: Node instance
        """

        self._nodes.append(node)

    def removeNode(self, node):
        """
        Removes a node from this module.

        :param node: Node instance
        """

        if node in self._nodes:
            self._nodes.remove(node)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """

        return self._settings

    def setSettings(self, settings):
        """
        Sets the module settings

        :param settings: module settings (dictionary)
        """

        self._settings.update(settings)
        self._saveSettings()

    def createNode(self, node_class, server, project):
        """
        Creates a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        :param project: Project instance
        """

        log.info("creating node {}".format(node_class))

        # create an instance of the node class
        return node_class(self, server, project)

    def setupNode(self, node, node_name):
        """
        Setups a node.

        :param node: Node instance
        :param node_name: Node name
        """

        log.info("configuring node {} with id {}".format(node, node.id()))

        vm = None
        if node_name:
            for vm_key, info in self._virtualbox_vms.items():
                if node_name == info["name"]:
                    vm = vm_key

        if not vm:
            selected_vms = []
            for vm, info in self._virtualbox_vms.items():
                if info["server"] == node.server().host() or (node.server().isLocal() and info["server"] == "local"):
                    selected_vms.append(vm)

            if not selected_vms:
                raise ModuleError("No VirtualBox VM on server {}".format(node.server().url()))
            elif len(selected_vms) > 1:

                from gns3.main_window import MainWindow
                mainwindow = MainWindow.instance()

                (selection, ok) = QtWidgets.QInputDialog.getItem(mainwindow, "VirtualBox VM", "Please choose a VM", selected_vms, 0, False)
                if ok:
                    vm = selection
                else:
                    raise ModuleError("Please select a VirtualBox VM")

            else:
                vm = selected_vms[0]

        linked_base = self._virtualbox_vms[vm]["linked_base"]
        if not linked_base:
            for other_node in self._nodes:
                if other_node.settings()["vmname"] == self._virtualbox_vms[vm]["vmname"] and \
                        (self._virtualbox_vms[vm]["server"] == "local" and other_node.server().isLocal() or self._virtualbox_vms[vm]["server"] == other_node.server().host):
                    raise ModuleError("Sorry a VirtualBox VM without the linked base setting enabled can only be used once in your topology")
        elif node.project().temporary():
            raise ModuleError("Sorry, VirtualBox linked clones are not supported in temporary projects")

        vm_settings = {}
        for setting_name, value in self._virtualbox_vms[vm].items():
            if setting_name != "name" and setting_name in node.settings() and value != "" and value is not None:
                vm_settings[setting_name] = value

        name = self._virtualbox_vms[vm]["name"]
        vmname = self._virtualbox_vms[vm]["vmname"]
        port_name_format = self._virtualbox_vms[vm]["port_name_format"]
        port_segment_size = self._virtualbox_vms[vm]["port_segment_size"]
        first_port_name = self._virtualbox_vms[vm]["first_port_name"]

        default_name_format = VBOX_VM_SETTINGS["default_name_format"]
        if self._virtualbox_vms[vm]["default_name_format"]:
            default_name_format = self._virtualbox_vms[vm]["default_name_format"]
        if linked_base:
            default_name_format = default_name_format.replace('{name}', name)
            name = None

        node.setup(vmname,
                   name=name,
                   port_name_format=port_name_format,
                   port_segment_size=port_segment_size,
                   first_port_name=first_port_name,
                   linked_clone=linked_base,
                   additional_settings=vm_settings,
                   default_name_format=default_name_format)

    def reset(self):
        """
        Resets the module.
        """

        log.info("VirtualBox module reset")
        self._nodes.clear()

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """

        if name in globals():
            return globals()[name]
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [VirtualBoxVM]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """

        nodes = []
        for vbox_vm in self._virtualbox_vms.values():
            nodes.append(
                {"class": VirtualBoxVM.__name__,
                 "name": vbox_vm["name"],
                 "server": vbox_vm["server"],
                 "symbol": vbox_vm["symbol"],
                 "categories": [vbox_vm["category"]]}
            )
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """

        from .pages.virtualbox_preferences_page import VirtualBoxPreferencesPage
        from .pages.virtualbox_vm_preferences_page import VirtualBoxVMPreferencesPage
        return [VirtualBoxPreferencesPage, VirtualBoxVMPreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of VirtualBox module.

        :returns: instance of VirtualBox
        """

        if not hasattr(VirtualBox, "_instance"):
            VirtualBox._instance = VirtualBox()
        return VirtualBox._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
To show a advanced message box.
"""

from gns3.qt import QtWidgets


def MessageBox(parent, title, message, details="", icon=QtWidgets.QMessageBox.Critical):

    msgbox = QtWidgets.QMessageBox(parent)
    msgbox.setWindowTitle(title)
    msgbox.setText(message)
    msgbox.setIcon(icon)
    if details:
        msgbox.setDetailedText(details)
    msgbox.exec_()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    dialog = MessageBox(main, "Test", "Hello world", details="A lot of details")
    dialog.show()
    exit_code = app.exec_()



#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys

from ..qt import QtWidgets, QtCore, QtGui, qpartial
from ..ui.appliance_wizard_ui import Ui_ApplianceWizard
from ..image_manager import ImageManager
from ..modules import Qemu
from ..registry.appliance import Appliance
from ..registry.registry import Registry
from ..registry.config import Config, ConfigException
from ..registry.image import Image
from ..utils import human_filesize
from ..utils.wait_for_lambda_worker import WaitForLambdaWorker
from ..utils.progress_dialog import ProgressDialog
from ..servers import Servers
from ..gns3_vm import GNS3VM
from ..local_config import LocalConfig


class ApplianceWizard(QtWidgets.QWizard, Ui_ApplianceWizard):

    def __init__(self, parent, path):
        super().__init__(parent)

        self._path = path
        self.setupUi(self)
        images_directories = list()
        images_directories.append(os.path.dirname(self._path))
        download_directory = QtCore.QStandardPaths.writableLocation(QtCore.QStandardPaths.DownloadLocation)
        if download_directory != "" and download_directory != os.path.dirname(self._path):
            images_directories.append(download_directory)
        self._registry = Registry(images_directories)
        self._appliance = Appliance(self._registry, self._path)
        self._registry.appendImageDirectory(os.path.join(ImageManager.instance().getDirectory(), self._appliance.image_dir_name()))

        self.uiApplianceVersionTreeWidget.currentItemChanged.connect(self._applianceVersionCurrentItemChangedSlot)
        self.uiRefreshPushButton.clicked.connect(self._refreshVersions)
        self.uiDownloadPushButton.clicked.connect(self._downloadPushButtonClickedSlot)
        self.uiImportPushButton.clicked.connect(self._importPushButtonClickedSlot)
        self.uiCreateVersionPushButton.clicked.connect(self._createVersionPushButtonClickedSlot)

        self.uiRemoteRadioButton.toggled.connect(self._remoteServerToggledSlot)
        if hasattr(self, "uiVMRadioButton"):
            self.uiVMRadioButton.toggled.connect(self._vmToggledSlot)

        self.uiLocalRadioButton.toggled.connect(self._localToggledSlot)
        self.uiServerWizardPage.isComplete = self._uiServerWizardPage_isComplete

    def initializePage(self, page_id):
        """
        Initialize Wizard pages.

        :param page_id: page identifier
        """
        super().initializePage(page_id)

        if self._appliance["category"] == "guest":
            symbol = ":/symbols/computer.svg"
        else:
            symbol = ":/symbols/{}.svg".format(self._appliance["category"])
        self.page(page_id).setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(symbol))

        if "qemu" in self._appliance:
            type = "qemu"
        elif "iou" in self._appliance:
            type = "iou"
        elif "docker" in self._appliance:
            type = "docker"
        elif "dynamips" in self._appliance:
            type = "dynamips"

        if self.page(page_id) == self.uiInfoWizardPage:
            self.uiInfoWizardPage.setTitle(self._appliance["product_name"])
            self.uiDescriptionLabel.setText(self._appliance["description"])

            info = (
                ("Category", "category"),
                ("Product", "product_name"),
                ("Vendor", "vendor_name"),
                ("Status", "status"),
                ("Maintainer", "maintainer"),
                ("Architecture", "qemu/arch"),
                ("KVM", "qemu/kvm")
            )

            self.uiInfoTreeWidget.clear()
            for (name, key) in info:
                if "/" in key:
                    key, subkey = key.split("/")
                    value = self._appliance.get(key, {}).get(subkey, None)
                else:
                    value = self._appliance.get(key, None)
                if value is None:
                    continue
                item = QtWidgets.QTreeWidgetItem([name + ":", value])
                font = item.font(0)
                font.setBold(True)
                item.setFont(0, font)
                self.uiInfoTreeWidget.addTopLevelItem(item)

        elif self.page(page_id) == self.uiServerWizardPage:
            self.uiRemoteServersComboBox.clear()
            for server in Servers.instance().remoteServers().values():
                self.uiRemoteServersComboBox.addItem(server.url(), server)

            if not GNS3VM.instance().isRunning():
                self.uiVMRadioButton.setEnabled(False)

            if (sys.platform.startswith("darwin") or sys.platform.startswith("win")):
                if type == "qemu":
                    # Qemu has issues on OSX and Windows we disallow usage of the local server
                    if not LocalConfig.instance().experimental():
                        self.uiLocalRadioButton.setEnabled(False)
                elif type != "dynamips":
                    self.uiLocalRadioButton.setEnabled(False)

            if GNS3VM.instance().isRunning():
                self.uiVMRadioButton.setChecked(True)
            elif Servers.instance().localServer().isLocalServerRunning() and self.uiLocalRadioButton.isEnabled():
                self.uiLocalRadioButton.setChecked(True)
            elif len(Servers.instance().remoteServers().values()) > 0:
                self.uiRemoteRadioButton.setChecked(True)
            else:
                self.uiRemoteRadioButton.setChecked(False)

        elif self.page(page_id) == self.uiFilesWizardPage:
            self._refreshVersions()

        elif self.page(page_id) == self.uiQemuWizardPage:
            Qemu.instance().getQemuBinariesFromServer(self._server, qpartial(self._getQemuBinariesFromServerCallback), [self._appliance["qemu"]["arch"]])

        elif self.page(page_id) == self.uiSummaryWizardPage:
            self.uiSummaryTreeWidget.clear()

            for key in self._appliance[type]:
                item = QtWidgets.QTreeWidgetItem([key.replace('_', ' ').capitalize() + ":", str(self._appliance[type][key])])
                font = item.font(0)
                font.setBold(True)
                item.setFont(0, font)
                self.uiSummaryTreeWidget.addTopLevelItem(item)
            self.uiSummaryTreeWidget.resizeColumnToContents(0)

        elif self.page(page_id) == self.uiUsageWizardPage:
            self.uiUsageTextEdit.setText("The appliance is available in the {} category. \n\n{}".format(
                self._appliance["category"].replace("_", " "),
                self._appliance.get("usage", ""))
            )

        elif self.page(page_id) == self.uiCheckServerWizardPage:
            self.uiCheckServerLabel.setText("Please wait while checking server capacities...")
            if 'qemu' in self._appliance:
                if self._appliance['qemu'].get('kvm', 'require') == 'require':
                    self._server_check = False # If the server as the capacities for running the appliance
                    Qemu.instance().getQemuCapabilitiesFromServer(self._server, qpartial(self._qemuServerCapabilitiesCallback))
                    return
            self.uiCheckServerLabel.setText("")
            self._server_check = True
            self.next()

    def _qemuServerCapabilitiesCallback(self, result, error=None, *args, **kwargs):
        """
        Check if server support KVM or not
        """
        if error is None and "kvm" in result and self._appliance["qemu"]["arch"] in result["kvm"]:
            self._server_check = True
            self.uiCheckServerLabel.setText("GNS3 server requirements is OK you can continue the installation")
        else:
            if error:
                msg = result["message"]
            else:
                msg = "The remote server doesn't support KVM. You need a Linux server or the GNS3 VM with VMware and CPU virtualization instructions."
            self.uiCheckServerLabel.setText(msg)
            QtWidgets.QMessageBox.critical(self, "Qemu", msg)
            self._server_check = False

    def _uiServerWizardPage_isComplete(self):
        return self.uiRemoteRadioButton.isEnabled() or self.uiVMRadioButton.isEnabled() or self.uiLocalRadioButton.isEnabled()

    def _refreshVersions(self):
        """
        Refresh the list of files for different version of the appliance
        """

        self.uiFilesWizardPage.setSubTitle("The following versions are available for " + self._appliance["product_name"] + ".  Check the status of files required to install.")
        self.uiApplianceVersionTreeWidget.clear()

        worker = WaitForLambdaWorker(lambda: self._resfreshDialogWorker())
        progress_dialog = ProgressDialog(worker, "Add appliance", "Scanning directories for files...", None, busy=True, parent=self)
        progress_dialog.show()
        if progress_dialog.exec_():
            for version in self._appliance["versions"]:
                top = QtWidgets.QTreeWidgetItem(["{} {}".format(self._appliance["product_name"], version["name"])])

                size = 0
                status = "Ready to install"
                for image in version["images"].values():
                    if image["status"] == "Missing":
                        status = "Missing files"

                    size += image.get("filesize", 0)
                    image_widget = QtWidgets.QTreeWidgetItem(
                        [
                            "",
                            image["filename"],
                            human_filesize(image.get("filesize", 0)),
                            image["status"],
                            image["version"],
                            image.get("md5sum", "")
                        ])

                    if image["status"] == "Missing":
                        image_widget.setForeground(3, QtGui.QBrush(QtGui.QColor("red")))
                    else:
                        image_widget.setForeground(3, QtGui.QBrush(QtGui.QColor("green")))

                    # Associated data stored are col 0: version, col 1: image
                    image_widget.setData(0, QtCore.Qt.UserRole, version)
                    image_widget.setData(1, QtCore.Qt.UserRole, image)
                    image_widget.setData(2, QtCore.Qt.UserRole, self._appliance)
                    top.addChild(image_widget)

                font = top.font(0)
                font.setBold(True)
                top.setFont(0, font)

                expand = True
                if status == "Missing files":
                    top.setForeground(3, QtGui.QBrush(QtGui.QColor("red")))
                else:
                    expand = False
                    top.setForeground(3, QtGui.QBrush(QtGui.QColor("green")))

                top.setData(2, QtCore.Qt.DisplayRole, human_filesize(size))
                top.setData(3, QtCore.Qt.DisplayRole, status)
                top.setData(2, QtCore.Qt.UserRole, self._appliance)
                top.setData(0, QtCore.Qt.UserRole, version)
                self.uiApplianceVersionTreeWidget.addTopLevelItem(top)
                if expand:
                    top.setExpanded(True)

            self.uiApplianceVersionTreeWidget.resizeColumnToContents(0)
            self.uiApplianceVersionTreeWidget.resizeColumnToContents(1)
            self.uiApplianceVersionTreeWidget.setCurrentItem(self.uiApplianceVersionTreeWidget.topLevelItem(0))

    def _resfreshDialogWorker(self):
        """
        Scan local directory in order to found the images on disk
        """

        # Docker do not have versions
        if not "versions" in self._appliance:
            return

        for version in self._appliance["versions"]:
            for image in version["images"].values():
                img = self._registry.search_image_file(image["filename"], image.get("md5sum"), image.get("filesize"))
                if img:
                    image["status"] = "Found"
                    image["md5sum"] = img.md5sum
                    image["filesize"] = img.filesize
                else:
                    image["status"] = "Missing"

    def _applianceVersionCurrentItemChangedSlot(self, current, previous):
        """
        Called when user select a different item in the list of appliance files
        """
        self.uiDownloadPushButton.hide()
        self.uiImportPushButton.hide()
        self.uiExplainDownloadLabel.hide()

        if current is None:
            return

        image = current.data(1, QtCore.Qt.UserRole)
        if image is not None:
            if "direct_download_url" in image or "download_url" in image:
                self.uiDownloadPushButton.show()
            self.uiImportPushButton.show()

    def _downloadPushButtonClickedSlot(self):
        """
        Called when user want to download an appliance images.
        He should have selected the file before.
        """
        current = self.uiApplianceVersionTreeWidget.currentItem()

        if current is None:
            return

        data = current.data(1, QtCore.Qt.UserRole)
        if data is not None:
            if "direct_download_url" in data:
                QtGui.QDesktopServices.openUrl(QtCore.QUrl(data["direct_download_url"]))
                if "compression" in data:
                    QtWidgets.QMessageBox.warning(self, "Add appliance", "The file is compressed with {} you need to uncompress it before using it.".format(data["compression"]))
            else:
                QtWidgets.QMessageBox.warning(self, "Add appliance", "Download will redirect you where the required file can be downloaded, you may have to be registered with the vendor in order to download the file.")
                QtGui.QDesktopServices.openUrl(QtCore.QUrl(data["download_url"]))

    def _createVersionPushButtonClickedSlot(self):
        """
        Allow user to create a new version of an appliance
        """

        new_version, ok = QtWidgets.QInputDialog.getText(self, "Creating a new version", "Creating a new version allows to import unknown files to use with this appliance.\nPlease share your experience on the GNS3 community if this version works.\n\nVersion name:", QtWidgets.QLineEdit.Normal)
        if ok:
            self._appliance.create_new_version(new_version)
            self._refreshVersions()

    def _importPushButtonClickedSlot(self):
        """
        Called when user want to import an appliance images.
        He should have selected the file before.
        """

        current = self.uiApplianceVersionTreeWidget.currentItem()
        disk = current.data(1, QtCore.Qt.UserRole)

        path, _ = QtWidgets.QFileDialog.getOpenFileName()
        if len(path) == 0:
            return

        image = Image(path)
        if "md5sum" in disk and image.md5sum != disk["md5sum"]:
            QtWidgets.QMessageBox.warning(self.parent(), "Add appliance", "This is not the correct file. The MD5 sum is {} and should be {}. For OVA you need to import the OVA/OVF not the file inside the archive.".format(image.md5sum, disk["md5sum"]))
            return

        config = Config()
        worker = WaitForLambdaWorker(lambda: image.copy(os.path.join(config.images_dir, self._appliance.image_dir_name()), disk["filename"]), allowed_exceptions=[OSError, ValueError])
        progress_dialog = ProgressDialog(worker, "Add appliance", "Importing the appliance...", None, busy=True, parent=self)
        if not progress_dialog.exec_():
            return
        self._refreshVersions()

    def _getQemuBinariesFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getQemuBinariesFromServer.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Qemu binaries", "{}".format(result["message"]))
        else:
            self.uiQemuListComboBox.clear()
            for qemu in result:
                if qemu["version"]:
                    self.uiQemuListComboBox.addItem("{path} (v{version})".format(path=qemu["path"], version=qemu["version"]), qemu["path"])
                else:
                    self.uiQemuListComboBox.addItem("{path}".format(path=qemu["path"]), qemu["path"])
            if self.uiQemuListComboBox.count() == 1:
                self.next()

    def _install(self, version):
        """
        Install the appliance to GNS3

        :params version: Version name
        """

        try:
            config = Config()
        except OSError as e:
            QtWidgets.QMessageBox.critical(self.parent(), "Add appliance", str(e))
            return False

        if version is None:
            appliance_configuration = self._appliance.copy()
        else:
            appliance_configuration = self._appliance.search_images_for_version(version)

        if self._server.isLocal():
            server_string = "local"
        elif self._server.isGNS3VM():
            server_string = "vm"
        else:
            server_string = self._server.url()

        while len(appliance_configuration["name"]) == 0 or not config.is_name_available(appliance_configuration["name"]):
            QtWidgets.QMessageBox.warning(self.parent(), "Add appliance", "The name \"{}\" is already used by another appliance".format(appliance_configuration["name"]))
            appliance_configuration["name"], ok = QtWidgets.QInputDialog.getText(self.parent(), "Add appliance", "New name:", QtWidgets.QLineEdit.Normal, appliance_configuration["name"])
            appliance_configuration["name"] = appliance_configuration["name"].strip()

        if "qemu" in appliance_configuration:
            appliance_configuration["qemu"]["path"] = self.uiQemuListComboBox.currentData()

        worker = WaitForLambdaWorker(lambda: config.add_appliance(appliance_configuration, server_string), allowed_exceptions=[ConfigException, OSError])
        progress_dialog = ProgressDialog(worker, "Add appliance", "Install the appliance...", None, busy=True, parent=self)
        progress_dialog.show()
        if not progress_dialog.exec_():
            return False

        worker = WaitForLambdaWorker(lambda: config.save(), allowed_exceptions=[ConfigException, OSError])
        progress_dialog = ProgressDialog(worker, "Add appliance", "Install the appliance...", None, busy=True, parent=self)
        progress_dialog.show()
        if progress_dialog.exec_():
            QtWidgets.QMessageBox.information(self.parent(), "Add appliance", "{} installed!".format(appliance_configuration["name"]))
            return True

    def nextId(self):
        if self.currentPage() == self.uiServerWizardPage:
            if "docker" in self._appliance:
                return super().nextId() + 3
            elif "qemu" not in self._appliance:
                return super().nextId() + 1
        elif self.currentPage() == self.uiFilesWizardPage:
            if "qemu" not in self._appliance:
                return super().nextId() + 1
        return super().nextId()

    def validateCurrentPage(self):
        """
        Validates the settings.
        """

        if self.currentPage() == self.uiFilesWizardPage:
            current = self.uiApplianceVersionTreeWidget.currentItem()
            version = current.data(0, QtCore.Qt.UserRole)
            appliance = current.data(2, QtCore.Qt.UserRole)
            if not self._appliance.is_version_installable(version["name"]):
                QtWidgets.QMessageBox.warning(self, "Appliance", "Sorry, you cannot install {} with missing files".format(appliance["name"]))
                return False
            reply = QtWidgets.QMessageBox.question(self, "Appliance", "Would you like to install {} version {}?".format(appliance["name"], version["name"]),
                                                   QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.No:
                return False

        elif self.currentPage() == self.uiUsageWizardPage:
            current = self.uiApplianceVersionTreeWidget.currentItem()
            if current:
                version = current.data(0, QtCore.Qt.UserRole)
                return self._install(version["name"])
            else:
                return self._install(None)

        elif self.currentPage() == self.uiServerWizardPage:
            if self.uiRemoteRadioButton.isChecked():
                if not Servers.instance().remoteServers():
                    QtWidgets.QMessageBox.critical(self, "Remote server", "There is no remote server registered in your preferences")
                    return False
                self._server = self.uiRemoteServersComboBox.itemData(self.uiRemoteServersComboBox.currentIndex())
            elif hasattr(self, "uiVMRadioButton") and self.uiVMRadioButton.isChecked():
                gns3_vm_server = Servers.instance().vmServer()
                if gns3_vm_server is None:
                    QtWidgets.QMessageBox.critical(self, "GNS3 VM", "The GNS3 VM is not running")
                    return False
                self._server = gns3_vm_server
            else:
                if (sys.platform.startswith("darwin") or sys.platform.startswith("win")):
                    if "qemu" in self._appliance:
                        reply = QtWidgets.QMessageBox.question(self, "Appliance", "Qemu on Windows and MacOSX is not supported by the GNS3 team. Are you sur to continue?", QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
                        if reply == QtWidgets.QMessageBox.No:
                            return False

                self._server = Servers.instance().localServer()

        elif self.currentPage() == self.uiQemuWizardPage:
            if self.uiQemuListComboBox.currentIndex() == -1:
                QtWidgets.QMessageBox.critical(self, "Qemu binary", "No compatible Qemu binary selected")
                return False

        elif self.currentPage() == self.uiCheckServerWizardPage:
            return self._server_check

        return True

    def _vmToggledSlot(self, checked):
        """
        Slot for when the VM radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    def _remoteServerToggledSlot(self, checked):
        """
        Slot for when the remote server radio button is toggled.

        :param checked: either the button is checked or not
        """

        if checked:
            self.uiRemoteServersGroupBox.setEnabled(True)
            self.uiRemoteServersGroupBox.show()

    def _localToggledSlot(self, checked):
        """
        Slot for when the local server radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Wizard for VirtualBox VMs.
"""

from gns3.qt import QtGui, QtWidgets
from gns3.servers import Servers
from gns3.dialogs.vm_wizard import VMWizard

from ..ui.virtualbox_vm_wizard_ui import Ui_VirtualBoxVMWizard
from .. import VirtualBox


class VirtualBoxVMWizard(VMWizard, Ui_VirtualBoxVMWizard):

    """
    Wizard to create a VirtualBox VM.

    :param virtualbox_vms: existing VirtualBox VMs
    :param parent: parent widget
    """

    def __init__(self, virtualbox_vms, parent):

        super().__init__(virtualbox_vms, VirtualBox.instance().settings()["use_local_server"], parent)
        self._virtualbox_vms = virtualbox_vms
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(":/icons/virtualbox.png"))

        if not Servers.instance().remoteServers():
            # skip the server page if we use the local server
            self.setStartId(1)

    def validateCurrentPage(self):
        """
        Validates the server.
        """

        if super().validateCurrentPage() is False:
            return False

        if self.currentPage() == self.uiVirtualBoxWizardPage:
            if not self.uiVMListComboBox.count():
                QtWidgets.QMessageBox.critical(self, "VirtualBox VMs", "There is no VirtualBox VM available!")
                return False
        return True

    def initializePage(self, page_id):

        super().initializePage(page_id)
        if self.page(page_id) == self.uiVirtualBoxWizardPage:
            self.uiVMListComboBox.clear()
            self._server.get("/virtualbox/vms", self._getVirtualBoxVMsFromServerCallback, progressText="Listing VirtualBox VMs...")

    def _getVirtualBoxVMsFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getVirtualBoxVMsFromServer.

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "VirtualBox VMs", "{}".format(result["message"]))
        else:
            self.uiVMListComboBox.clear()
            existing_vms = []
            for existing_vm in self._virtualbox_vms.values():
                existing_vms.append(existing_vm["vmname"])

            for vm in result:
                if vm["vmname"] not in existing_vms:
                    self.uiVMListComboBox.addItem(vm["vmname"], vm)

    def getSettings(self):
        """
        Returns the settings set in this Wizard.

        :return: settings dict
        """

        if self.uiLocalRadioButton.isChecked():
            server = "local"
        else:
            server = self.uiRemoteServersComboBox.currentText()

        index = self.uiVMListComboBox.currentIndex()
        vmname = self.uiVMListComboBox.itemText(index)
        vminfo = self.uiVMListComboBox.itemData(index)

        settings = {
            "name": vmname,
            "vmname": vmname,
            "server": server,
            "ram": vminfo["ram"],
            "linked_base": self.uiBaseVMCheckBox.isChecked()
        }

        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Dynamips bridge implementation  on the client side (in the form of a hub).
Asynchronously sends JSON messages to the GNS3 server and receives responses with callbacks.
"""

from gns3.node import Node
from gns3.ports.ethernet_port import EthernetPort
from .device import Device

import logging
log = logging.getLogger(__name__)


class EthernetHub(Device):

    """
    Dynamips Ethernet hub.

    :param module: parent module for this node
    :param server: GNS3 server instance
    :param project: Project instance
    """

    def __init__(self, module, server, project):

        super().__init__(module, server, project)
        self.setStatus(Node.started)  # this is an always-on node
        self._ports = []
        self._settings = {"name": "",
                          "ports": []}

    def setup(self, name=None, device_id=None, initial_ports=[]):
        """
        Setups this hub.

        :param name: optional name for this hub
        :param device_id: device identifier on the server
        :param initial_ports: ports to automatically be added when creating this hub
        """

        # let's create a unique name if none has been chosen
        if not name:
            name = self.allocateName("HUB")

        if not name:
            self.error_signal.emit(self.id(), "could not allocate a name for this Ethernet hub")
            return

        self._settings["name"] = name
        if not initial_ports:
            # default configuration if no initial ports
            for port_number in range(1, 9):
                # add 8 ports
                initial_ports.append({"name": str(port_number),
                                      "port_number": port_number})

        # add the initial ports
        for initial_port in initial_ports:
            port = EthernetPort(initial_port["name"])
            port.setPortNumber(initial_port["port_number"])
            if "id" in initial_port:
                port.setId(initial_port["id"])
            port.setStatus(EthernetPort.started)
            port.setPacketCaptureSupported(True)
            self._ports.append(port)
            self._settings["ports"].append(port.portNumber())

        params = {"name": name,
                  "device_type": "ethernet_hub"}
        if device_id:
            params["device_id"] = device_id
        self.httpPost("/dynamips/devices", self._setupCallback, body=params)

    def update(self, new_settings):
        """
        Updates the settings for this Ethernet hub.

        :param new_settings: settings dictionary
        """

        updated = False
        if "ports" in new_settings:
            ports_to_create = []
            ports = new_settings["ports"]
            for port_number in ports:
                if port_number not in ports_to_create:
                    ports_to_create.append(str(port_number))

            for port in self._ports.copy():
                if port.isFree():
                    self._ports.remove(port)
                    updated = True
                    log.debug("port {} has been removed".format(port.name()))
                elif port.name() in ports_to_create:
                    ports_to_create.remove(port.name())

            for port_name in ports_to_create:
                port = EthernetPort(port_name)
                port.setPortNumber(int(port_name))
                port.setStatus(EthernetPort.started)
                port.setPacketCaptureSupported(True)
                self._ports.append(port)
                updated = True
                log.debug("port {} has been added".format(port_name))

            self._settings["ports"] = new_settings["ports"].copy()

        params = {}
        if "name" in new_settings and new_settings["name"] != self.name():
            if self.hasAllocatedName(new_settings["name"]):
                self.error_signal.emit(self.id(), 'Name "{}" is already used by another node'.format(new_settings["name"]))
                return
            params["name"] = new_settings["name"]
            updated = True

        if updated:
            if params:
                log.debug("{} is being updated: {}".format(self.name(), params))
                self.httpPut("/dynamips/devices/{device_id}".format(device_id=self._device_id), self._updateCallback, body=params)
            else:
                log.info("{} has been updated".format(self.name()))
                self.updated_signal.emit()

    def _updateCallback(self, result, error=False, **kwargs):
        """
        Callback for update.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while updating {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            if "name" in result:
                self._settings["name"] = result["name"]
                self.updateAllocatedName(result["name"])
            log.info("{} has been updated".format(self.name()))
            self.updated_signal.emit()

    def addNIO(self, port, nio):
        """
        Adds a new NIO on the specified port for this Ethernet hub.

        :param port: Port instance
        :param nio: NIO instance
        """

        params = {}
        params["nio"] = self.getNIOInfo(nio)
        log.debug("{} is adding an {}: {}".format(self.name(), nio, params))
        self.httpPost("/{prefix}/devices/{device_id}/ports/{port}/nio".format(
            port=port.portNumber(),
            prefix=self.URL_PREFIX,
            device_id=self._device_id),
            self._addNIOCallback,
            context={"port_id": port.id()},
            body=params)

    def info(self):
        """
        Returns information about this Ethernet hub.

        :returns: formated string
        """

        info = """Ethernet hub {name} is always-on
  Local node ID is {id}
  Server's Device ID is {device_id}
  Hardware is Dynamips emulated simple Ethernet hub
  Hub's server runs on {host}:{port}
""".format(name=self.name(),
           id=self.id(),
           device_id=self._device_id,
           host=self._server.host(),
           port=self._server.port())

        port_info = ""
        for port in self._ports:
            if port.isFree():
                port_info += "   Port {} is empty\n".format(port.name())
            else:
                port_info += "   Port {name} {description}\n".format(name=port.name(),
                                                                     description=port.description())

        return info + port_info

    def dump(self):
        """
        Returns a representation of this Ethernet hub
        (to be saved in a topology file)

        :returns: representation of the node (dictionary)
        """

        hub = {"id": self.id(),
               "device_id": self._device_id,
               "type": self.__class__.__name__,
               "description": str(self),
               "properties": {"name": self.name()},
               "server_id": self._server.id()}

        # add the ports
        if self._ports:
            ports = hub["ports"] = []
            for port in self._ports:
                ports.append(port.dump())

        return hub

    def load(self, node_info):
        """
        Loads an Ethernet hub representation
        (from a topology file).

        :param node_info: representation of the node (dictionary)
        """

        settings = node_info["properties"]
        name = settings.pop("name")

        # pre-1.3 projects have no device id, set to 1 to have
        # a proper project conversion on the server side
        device_id = node_info.get("device_id", 1)

        # create the ports with the correct port numbers and IDs
        ports = []
        if "ports" in node_info:
            ports = node_info["ports"]

        log.info("Ethernet hub {} is loading".format(name))
        self.setName(name)
        self.setup(name, device_id, ports)

    def name(self):
        """
        Returns the name of this hub.

        :returns: name (string)
        """

        return self._settings["name"]

    def settings(self):
        """
        Returns all this hub settings.

        :returns: settings dictionary
        """

        return self._settings

    def ports(self):
        """
        Returns all the ports for this hub.

        :returns: list of Port instances
        """

        return self._ports

    def configPage(self):
        """
        Returns the configuration page widget to be used by the node properties dialog.

        :returns: QWidget object
        """

        from ..pages.ethernet_hub_configuration_page import EthernetHubConfigurationPage
        return EthernetHubConfigurationPage

    @staticmethod
    def defaultSymbol():
        """
        Returns the default symbol path for this node.

        :returns: symbol path (or resource).
        """

        return ":/symbols/hub.svg"

    @staticmethod
    def symbolName():

        return "Ethernet hub"

    @staticmethod
    def categories():
        """
        Returns the node categories the node is part of (used by the device panel).

        :returns: list of node category (integer)
        """

        return [Node.switches]

    def __str__(self):

        return "Ethernet hub"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/dynamips/ui/ethernet_hub_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ethernetHubConfigPageWidget(object):

    def setupUi(self, ethernetHubConfigPageWidget):
        ethernetHubConfigPageWidget.setObjectName("ethernetHubConfigPageWidget")
        ethernetHubConfigPageWidget.resize(381, 270)
        self.gridlayout = QtWidgets.QGridLayout(ethernetHubConfigPageWidget)
        self.gridlayout.setObjectName("gridlayout")
        self.uiSettingsGroupBox = QtWidgets.QGroupBox(ethernetHubConfigPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSettingsGroupBox.sizePolicy().hasHeightForWidth())
        self.uiSettingsGroupBox.setSizePolicy(sizePolicy)
        self.uiSettingsGroupBox.setObjectName("uiSettingsGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiSettingsGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiSettingsGroupBox)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiPortsLabel = QtWidgets.QLabel(self.uiSettingsGroupBox)
        self.uiPortsLabel.setObjectName("uiPortsLabel")
        self.gridLayout.addWidget(self.uiPortsLabel, 1, 0, 1, 1)
        self.uiPortsSpinBox = QtWidgets.QSpinBox(self.uiSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPortsSpinBox.sizePolicy().hasHeightForWidth())
        self.uiPortsSpinBox.setSizePolicy(sizePolicy)
        self.uiPortsSpinBox.setMinimum(0)
        self.uiPortsSpinBox.setMaximum(65535)
        self.uiPortsSpinBox.setProperty("value", 1)
        self.uiPortsSpinBox.setObjectName("uiPortsSpinBox")
        self.gridLayout.addWidget(self.uiPortsSpinBox, 1, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 71, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 1, 1, 1)
        self.gridlayout.addWidget(self.uiSettingsGroupBox, 0, 1, 1, 1)

        self.retranslateUi(ethernetHubConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(ethernetHubConfigPageWidget)

    def retranslateUi(self, ethernetHubConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        ethernetHubConfigPageWidget.setWindowTitle(_translate("ethernetHubConfigPageWidget", "Ethernet hub"))
        self.uiSettingsGroupBox.setTitle(_translate("ethernetHubConfigPageWidget", "Settings"))
        self.uiNameLabel.setText(_translate("ethernetHubConfigPageWidget", "Name:"))
        self.uiPortsLabel.setText(_translate("ethernetHubConfigPageWidget", "Number of ports:"))



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import zipfile
import shutil

from ..qt import QtCore
from ..servers import Servers


class ExportProjectWorker(QtCore.QObject):
    """
    Export the current topology to a portable format
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    def __init__(self, project, path, include_images):
        super().__init__()
        self._project = project
        self._include_images = include_images
        self._path = path

    def run(self):

        vm_server = None
        for server in self._project.servers():
            if server.isGNS3VM():
                vm_server = server

        if vm_server:
            self._project.get(vm_server,
                              "/export",
                              self._exportVmReceived,
                              downloadProgressCallback=self._downloadFileProgress)
        else:
            self._project.get(Servers.instance().localServer(),
                              "/export?include_images={}".format(self._include_images),
                              self._exportLocalReceived,
                              downloadProgressCallback=self._downloadFileProgress)

    def _exportVmReceived(self, content, error=False, server=None, context={}, **kwargs):
        if error:
            self.error.emit("Can't export the project from the VM", True)
            self.finished.emit()
            return

        vm_path = os.path.join(self._project.filesDir(), "servers", "vm")
        if os.path.exists(vm_path):
            shutil.rmtree(vm_path, ignore_errors=True)
        try:
            os.makedirs(vm_path, exist_ok=True)
        except OSError as e:
            self.error.emit("Can't create directory {}: {}".format(vm_path, e), True)
            self.finished.emit()
            return

        with zipfile.ZipFile(self._path) as myzip:
            myzip.extractall(vm_path)

        # We reset the content of the file
        try:
            open(self._path, 'wb+').close()
        except OSError as e:
            self.error.emit("Can't write project file {}: {}".format(self._path, e), True)
            self.finished.emit()
            return

        self._project.get(Servers.instance().localServer(), "/export?include_images={}".format(self._include_images), self._exportLocalReceived, downloadProgressCallback=self._downloadFileProgress)

    def _exportLocalReceived(self, content, error=False, server=None, context={}, **kwargs):
        if error:
            self.error.emit("Can't export the project from the local server", True)
            self.finished.emit()
            return
        self.finished.emit()

    def _downloadFileProgress(self, content, server=None, context={}, **kwargs):
        """
        Called for each part of the file
        """
        try:
            with open(self._path, 'ab') as f:
                f.write(content)
        except OSError as e:
            self.error.emit("Can't write project file {}: {}".format(self._path, e), True)
            self.finished.emit()
            return

    def cancel(self):
        pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.qt import QtWidgets, QtCore
from gns3.ui.new_appliance_dialog_ui import Ui_NewApplianceDialog
from gns3.dialogs.preferences_dialog import PreferencesDialog


import logging
log = logging.getLogger(__name__)


class NewApplianceDialog(QtWidgets.QDialog, Ui_NewApplianceDialog):
    """
    This dialog allow user to create a new appliance by opening
    the correct creation dialog
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self.uiImportApplianceTemplatePushButton.clicked.connect(self._importApplianceTemplatePushButtonClickedSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Ok).clicked.connect(self._okButtonClickedSlot)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Cancel).clicked.connect(self.reject)
        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Help).clicked.connect(self._helpButtonClickedSlot)

    def _importApplianceTemplatePushButtonClickedSlot(self):

        self.accept()
        from gns3.main_window import MainWindow
        MainWindow.instance().openApplianceActionSlot()

    def _okButtonClickedSlot(self):

        self.accept()
        dialog = PreferencesDialog(self.parent())
        if self.uiAddIOSRouterRadioButton.isChecked():
            self._setPreferencesPane(dialog, "Dynamips").uiNewIOSRouterPushButton.clicked.emit(False)
        elif self.uiAddIOUDeviceRadioButton.isChecked():
            self._setPreferencesPane(dialog, "IOS on UNIX").uiNewIOUDevicePushButton.clicked.emit(False)
        elif self.uiAddQemuVMRadioButton.isChecked():
            self._setPreferencesPane(dialog, "QEMU").uiNewQemuVMPushButton.clicked.emit(False)
        elif self.uiAddVirtualBoxVMRadioButton.isChecked():
            self._setPreferencesPane(dialog, "VirtualBox").uiNewVirtualBoxVMPushButton.clicked.emit(False)
        elif self.uiAddVMwareVMRadioButton.isChecked():
            self._setPreferencesPane(dialog, "VMware").uiNewVMwareVMPushButton.clicked.emit(False)
        elif self.uiAddDockerVMRadioButton.isChecked():
            self._setPreferencesPane(dialog, "Docker").uiNewDockerVMPushButton.clicked.emit(False)
        else:
            return
        dialog.exec_()

    def _helpButtonClickedSlot(self):

        help_text = """<html><p>This dialog helps you to add an appliance template in GNS3. In all cases you must provide your own images.</p>
        <p>You can download appliance template files (.gns3appliance) from <a href="https://gns3.com/marketplace/appliances">the GNS3 website</a></p>
        <p>A template file provides community tested settings to run a specific appliance in GNS3.</p></html>
        """
        QtWidgets.QMessageBox.information(self, "Help for adding a new appliance template", help_text)

    def _setPreferencesPane(self, dialog, name):
        """
        Finds the first child of the QTreeWidgetItem name.

        :param dialog: PreferencesDialog instance
        :param name: QTreeWidgetItem name

        :returns: current QWidget
        """

        pane = dialog.uiTreeWidget.findItems(name, QtCore.Qt.MatchFixedString)[0]
        child_pane = pane.child(0)
        dialog.uiTreeWidget.setCurrentItem(child_pane)
        return dialog.uiStackedWidget.currentWidget()

if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    dialog = NewApplianceDialog(main, console=True)
    dialog.show()
    exit_code = app.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/vpcs/ui/vpcs_device_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VPCSDeviceConfigPageWidget(object):

    def setupUi(self, VPCSDeviceConfigPageWidget):
        VPCSDeviceConfigPageWidget.setObjectName("VPCSDeviceConfigPageWidget")
        VPCSDeviceConfigPageWidget.resize(391, 246)
        self.gridLayout = QtWidgets.QGridLayout(VPCSDeviceConfigPageWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLineEdit = QtWidgets.QLineEdit(VPCSDeviceConfigPageWidget)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiConsolePortSpinBox = QtWidgets.QSpinBox(VPCSDeviceConfigPageWidget)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName("uiConsolePortSpinBox")
        self.gridLayout.addWidget(self.uiConsolePortSpinBox, 2, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 212, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 3, 1, 1, 1)
        self.uiConsolePortLabel = QtWidgets.QLabel(VPCSDeviceConfigPageWidget)
        self.uiConsolePortLabel.setObjectName("uiConsolePortLabel")
        self.gridLayout.addWidget(self.uiConsolePortLabel, 2, 0, 1, 1)
        self.uiNameLabel = QtWidgets.QLabel(VPCSDeviceConfigPageWidget)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)

        self.retranslateUi(VPCSDeviceConfigPageWidget)
        QtCore.QMetaObject.connectSlotsByName(VPCSDeviceConfigPageWidget)

    def retranslateUi(self, VPCSDeviceConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VPCSDeviceConfigPageWidget.setWindowTitle(_translate("VPCSDeviceConfigPageWidget", "VPCS device configuration"))
        self.uiConsolePortLabel.setText(_translate("VPCSDeviceConfigPageWidget", "Console port:"))
        self.uiNameLabel.setText(_translate("VPCSDeviceConfigPageWidget", "Name:"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import unittest.mock

from gns3.qt import QtCore, QtNetwork, FakeQtSignal
from gns3.http_client import HTTPClient
from gns3.version import __version__, __version_info__


@pytest.fixture
def network_manager(response):

    mock = unittest.mock.MagicMock()
    mock.sendCustomRequest.return_value = response
    return mock


@pytest.fixture
def response():
    response = unittest.mock.MagicMock()
    type(response).finished = unittest.mock.PropertyMock(return_value=FakeQtSignal())
    response.error.return_value = QtNetwork.QNetworkReply.NoError
    response.attribute.return_value = 200
    response.header.return_value = "application/json"
    return response


@pytest.fixture
def http_client(http_request, network_manager):

    return HTTPClient({"protocol": "http", "host": "127.0.0.1", "port": "3080"}, network_manager)


@pytest.yield_fixture(autouse=True)
def http_request():

    mock = unittest.mock.Mock()

    def call_request(url):
        mock(url)
        return mock
    with unittest.mock.patch("gns3.http_client.HTTPClient._request", side_effect=call_request):
        yield mock


def test_get_connected(http_client, http_request, network_manager, response):

    http_client._connected = True
    callback = unittest.mock.MagicMock()

    http_client.get("/test", callback)
    http_request.assert_called_with(QtCore.QUrl("http://127.0.0.1:3080/v1/test"))

    http_request.setRawHeader.assert_any_call(b"Content-Type", b"application/json")
    http_request.setRawHeader.assert_any_call(b"User-Agent", "GNS3 QT Client v{version}".format(version=__version__).encode())
    assert network_manager.sendCustomRequest.called
    args, kwargs = network_manager.sendCustomRequest.call_args
    assert args[0] == http_request
    assert args[1] == b"GET"

    # Trigger the completion
    response.finished.emit()

    assert callback.called


def test_get_connected_auth(http_client, http_request, network_manager, response):

    http_client._connected = True
    http_client._user = "gns3"
    http_client._password = "3sng"
    callback = unittest.mock.MagicMock()

    http_client.get("/test", callback)
    http_request.assert_called_with(QtCore.QUrl("http://gns3@127.0.0.1:3080/v1/test"))
    http_request.setRawHeader.assert_any_call(b"Content-Type", b"application/json")
    http_request.setRawHeader.assert_any_call(b"Authorization", b"Basic Z25zMzozc25n")
    http_request.setRawHeader.assert_any_call(b"User-Agent", "GNS3 QT Client v{version}".format(version=__version__).encode())
    assert network_manager.sendCustomRequest.called
    args, kwargs = network_manager.sendCustomRequest.call_args
    assert args[0] == http_request
    assert args[1] == b"GET"

    # Trigger the completion
    response.finished.emit()

    assert callback.called


def test_post_not_connected(http_client, http_request, network_manager, response):

    http_client._connected = False
    callback = unittest.mock.MagicMock()

    http_client.post("/test", callback, context={"toto": 42})

    args, kwargs = network_manager.sendCustomRequest.call_args
    assert args[0] == http_request
    assert args[1] == b"GET"

    response.header.return_value = "application/json"
    response.readAll.return_value = ("{\"version\": \"" + __version__ + "\", \"local\": true}").encode()

    # Trigger the completion of /version
    response.finished.emit()

    # Trigger the completion
    response.finished.emit()

    args, kwargs = network_manager.sendCustomRequest.call_args
    assert args[0] == http_request
    assert args[1] == b"POST"

    assert http_client._connected
    assert callback.called

    args, kwargs = callback.call_args
    assert kwargs["context"]["toto"] == 42


def test_post_not_connected_connection_failed(http_client, http_request, network_manager, response):

    http_client._connected = False
    callback = unittest.mock.MagicMock()

    response.error.return_value = QtNetwork.QNetworkReply.ConnectionRefusedError

    http_client.post("/test", callback)

    args, kwargs = network_manager.sendCustomRequest.call_args
    assert args[0] == http_request
    assert args[1] == b"GET"

    # Trigger the completion of /version
    response.finished.emit()

    assert callback.called


def test_progress_callback(http_client, response):

    http_client._connected = True
    callback = unittest.mock.MagicMock()
    progress = unittest.mock.MagicMock()

    http_client.setProgressCallback(progress)
    http_client.post("/test", callback)

    # Trigger the completion
    response.finished.emit()

    assert progress.add_query_signal.emit.called
    assert progress.remove_query_signal.emit.called


def test_processDownloadProgress(http_client):

    callback = unittest.mock.MagicMock()
    response = unittest.mock.MagicMock()
    response.header.return_value = "application/json"
    response.error.return_value = QtNetwork.QNetworkReply.NoError
    response.attribute.return_value = 200

    response.readAll.return_value = b'{"action": "ping"}'

    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert callback.called
    args, kwargs = callback.call_args
    assert args[0] == {"action": "ping"}


def test_processDownloadProgressHTTPError(http_client):

    callback = unittest.mock.MagicMock()
    response = unittest.mock.MagicMock()
    response.header.return_value = "application/json"
    response.error.return_value = QtNetwork.QNetworkReply.NoError
    response.attribute.return_value = 404

    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert not callback.called


def test_processDownloadProgressConnectionRefusedError(http_client):

    callback = unittest.mock.MagicMock()
    response = unittest.mock.MagicMock()
    response.header.return_value = "application/json"
    response.error.return_value = QtNetwork.QNetworkReply.ConnectionRefusedError
    response.attribute.return_value = 200

    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert not callback.called


def test_processDownloadProgressPartialJSON(http_client):
    """
    We can read an incomplete JSON on the network and we need
    to wait for the next part"""
    callback = unittest.mock.MagicMock()
    response = unittest.mock.MagicMock()
    response.header.return_value = "application/json"
    response.readAll.return_value = b'{"action": "ping"'
    response.error.return_value = QtNetwork.QNetworkReply.NoError
    response.attribute.return_value = 200

    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert not callback.called

    response.readAll.return_value = b'}\n{"a": "b"'
    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert callback.call_count == 1
    args, kwargs = callback.call_args
    assert args[0] == {"action": "ping"}


def test_processDownloadProgressPartialBytes(http_client):
    callback = unittest.mock.MagicMock()
    response = unittest.mock.MagicMock()
    response.header.return_value = "application/octet-stream"
    response.readAll.return_value = b'hello'
    response.error.return_value = QtNetwork.QNetworkReply.NoError
    response.attribute.return_value = 200

    http_client._processDownloadProgress(response, callback, {"query_id": "bla"}, 10, 100)

    assert callback.call_count == 1
    args, kwargs = callback.call_args
    assert args[0] == b'hello'


def test_dump(http_client):

    assert http_client.dump() == {
        'host': '127.0.0.1',
        'id': 0,
        'local': True,
        'port': 3080,
        'protocol': 'http',
        'vm': False}


def test_callbackConnect_version_ok(http_client):

    params = {
        "local": True,
        "version": __version__
    }
    http_client._callbackConnect("GET", "/version", None, {}, {}, params)
    assert http_client._connected


def test_callbackConnect_version_non_local(http_client):

    params = {
        "local": False,
        "version": __version__
    }
    mock = unittest.mock.MagicMock()
    http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
    assert http_client._connected is False
    mock.assert_called_with({"message": "Running server is not a GNS3 local server (not started with --local)"}, error=True, server=http_client)


def test_callbackConnect_version_non_local_remote_server(http_client):

    params = {
        "local": False,
        "version": __version__
    }
    mock = unittest.mock.MagicMock()
    http_client._local = False
    http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
    assert http_client._connected is True


def test_callbackConnect_major_version_invalid(http_client):

    params = {
        "local": True,
        "version": "1.2.3"
    }
    mock = unittest.mock.MagicMock()
    http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
    assert http_client._connected is False
    mock.assert_called_with({"message": "Client version {} differs with server version 1.2.3".format(__version__)}, error=True, server=http_client)


def test_callbackConnect_minor_version_invalid(http_client):

    new_version = "{}.{}.{}".format(__version_info__[0], __version_info__[1], __version_info__[2] + 1)
    params = {
        "local": True,
        "version": new_version
    }
    mock = unittest.mock.MagicMock()

    # Stable release
    if __version_info__[3] == 0:
        http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
        assert http_client._connected is False
        mock.assert_called_with({"message": "Client version {} differs with server version {}".format(__version__, new_version)}, error=True, server=http_client)
    else:
        http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
        assert http_client._connected is True


def test_callbackConnect_non_gns3_server(http_client):

    params = {
        "virus": True,
    }
    mock = unittest.mock.MagicMock()
    http_client._callbackConnect("GET", "/version", mock, {}, {}, params)
    assert http_client._connected is False
    mock.assert_called_with({"message": "The remote server http://127.0.0.1:3080 is not a GNS3 server"}, error=True, server=http_client)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys

from gns3.qt import QtWidgets
from gns3.servers import Servers
from gns3.gns3_vm import GNS3VM


class VMWizard(QtWidgets.QWizard):
    """
    Base class for VM wizard.

    :param devices: List of existing device for this type
    :param use_local_server: Value the use_local_server settings for this module
    :param parent: parent widget
    """

    def __init__(self, devices, use_local_server, parent):
        super().__init__(parent)
        self.setupUi(self)

        self.setModal(True)

        self._devices = devices
        self._use_local_server = use_local_server

        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        self.uiRemoteRadioButton.toggled.connect(self._remoteServerToggledSlot)
        if hasattr(self, "uiVMRadioButton"):
            self.uiVMRadioButton.toggled.connect(self._vmToggledSlot)

        self.uiLocalRadioButton.toggled.connect(self._localToggledSlot)

        # By default we use the local server
        self._server = Servers.instance().localServer()
        self.uiLocalRadioButton.setChecked(True)
        self._localToggledSlot(True)

        if Servers.instance().isNonLocalServerConfigured() is False:
            # skip the server page if we use the local server
            self.setStartId(1)

    def _vmToggledSlot(self, checked):
        """
        Slot for when the VM radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    def _remoteServerToggledSlot(self, checked):
        """
        Slot for when the remote server radio button is toggled.

        :param checked: either the button is checked or not
        """

        if checked:
            self.uiRemoteServersGroupBox.setEnabled(True)
            self.uiRemoteServersComboBox.setEnabled(True)
            self.uiRemoteServersGroupBox.show()

    def _localToggledSlot(self, checked):
        """
        Slot for when the local server radio button is toggled.

        :param checked: either the button is checked or not
        """
        if checked:
            self.uiRemoteServersGroupBox.setEnabled(False)
            self.uiRemoteServersGroupBox.hide()

    def setStartId(self, index):
        """
        Which page should we use when starting the Wizard
        """
        super().setStartId(index)
        # If we skip the initial page (choosing a server)
        # we check the settings
        if index != 0:
            self.uiLocalRadioButton.setChecked(True)

    def initializePage(self, page_id):

        if self.page(page_id) == self.uiServerWizardPage:
            self.uiRemoteServersComboBox.clear()

            if len(Servers.instance().remoteServers().values()) == 0:
                self.uiRemoteRadioButton.setEnabled(False)
            else:
                for server in Servers.instance().remoteServers().values():
                    self.uiRemoteServersComboBox.addItem(server.url(), server)

            if hasattr(self, "uiVMRadioButton") and not GNS3VM.instance().isRunning():
                self.uiVMRadioButton.setEnabled(False)
            if hasattr(self, "uiVMRadioButton") and GNS3VM.instance().isRunning():
                self.uiVMRadioButton.setChecked(True)
            elif self._use_local_server and self.uiLocalRadioButton.isEnabled():
                self.uiLocalRadioButton.setChecked(True)
            else:
                if self.uiRemoteRadioButton.isEnabled():
                    self.uiRemoteRadioButton.setChecked(True)
                else:
                    self.uiLocalRadioButton.setChecked(True)

    def validateCurrentPage(self):
        """
        Validates the server.
        """

        if hasattr(self, "uiNamePlatformWizardPage") and self.currentPage() == self.uiNamePlatformWizardPage:
            name = self.uiNameLineEdit.text()
            for device in self._devices.values():
                if device["name"] == name:
                    QtWidgets.QMessageBox.critical(self, "Name", "{} is already used, please choose another name".format(name))
                    return False
        elif self.currentPage() == self.uiServerWizardPage:
            if self.uiRemoteRadioButton.isChecked():
                if not Servers.instance().remoteServers():
                    QtWidgets.QMessageBox.critical(self, "Remote server", "There is no remote server registered in your preferences")
                    return False
                self._server = self.uiRemoteServersComboBox.itemData(self.uiRemoteServersComboBox.currentIndex())
            elif hasattr(self, "uiVMRadioButton") and self.uiVMRadioButton.isChecked():
                gns3_vm_server = Servers.instance().vmServer()
                if gns3_vm_server is None:
                    QtWidgets.QMessageBox.critical(self, "GNS3 VM", "The GNS3 VM is not running")
                    return False
                self._server = gns3_vm_server
            else:
                self._server = Servers.instance().localServer()
        return True



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Docker module implementation."""

from gns3.qt import QtWidgets
from gns3.local_config import LocalConfig

from ..module import Module
from ..module_error import ModuleError
from .docker_vm import DockerVM
from .settings import DOCKER_SETTINGS, DOCKER_CONTAINER_SETTINGS

import logging
log = logging.getLogger(__name__)


class Docker(Module):
    """Docker module."""

    def __init__(self):
        super().__init__()

        self._settings = {}
        self._docker_containers = {}
        self._nodes = []

        # load the settings
        self._loadSettings()

    def configChangedSlot(self):
        # load the settings
        self._loadSettings()

    def _saveSettings(self):
        """Saves the settings to the persistent settings file."""
        LocalConfig.instance().saveSectionSettings(
            self.__class__.__name__, self._settings)

    def _loadSettings(self):
        """Loads the settings from the persistent settings file."""
        local_config = LocalConfig.instance()
        self._settings = local_config.loadSectionSettings(
            self.__class__.__name__, DOCKER_SETTINGS)

        if "containers" in self._settings:
            for image in self._settings["containers"]:
                name = image.get("name")
                server = image.get("server")
                key = "{server}:{name}".format(server=server, name=name)
                if key in self._docker_containers or not name or not server:
                    continue
                container_settings = DOCKER_CONTAINER_SETTINGS.copy()
                container_settings.update(image)
                self._docker_containers[key] = container_settings

    def _saveDockerImages(self):
        """Saves the Docker containers to the persistent settings file."""

        self._settings["containers"] = list(self._docker_containers.values())
        self._saveSettings()

    def VMs(self):
        """
        Returns Docker images settings.

        :returns: Docker images settings
        :rtype: dict
        """

        return self._docker_containers

    def setVMs(self, new_docker_containers):
        """Sets Docker image settings.

        :param new_iou_images: Docker images settings (dictionary)
        """
        self._docker_containers = new_docker_containers.copy()
        self._saveDockerImages()

    @staticmethod
    def vmConfigurationPage():
        from .pages.docker_vm_configuration_page import DockerVMConfigurationPage
        return DockerVMConfigurationPage

    def addNode(self, node):
        """Adds a node to this module.

        :param node: Node instance
        """
        self._nodes.append(node)

    def removeNode(self, node):
        """Removes a node from this module.

        :param node: Node instance
        """
        if node in self._nodes:
            self._nodes.remove(node)

    def settings(self):
        """
        Returns the module settings

        :returns: module settings (dictionary)
        """
        return self._settings

    def setSettings(self, settings):
        """Sets the module settings

        :param settings: module settings (dictionary)
        """
        self._settings.update(settings)
        self._saveSettings()

    def createNode(self, node_class, server, project):
        """Creates a new node.

        :param node_class: Node object
        :param server: HTTPClient instance
        """
        log.info("creating node {}".format(node_class))
        # create an instance of the node class
        return node_class(self, server, project)

    def setupNode(self, node, node_name):
        """Sets up a node.

        :param node: Node instance
        :param node_name: Node name
        """
        log.info("configuring node {} with id {}".format(node, node.id()))

        image = None
        if node_name:
            for image_key, info in self._docker_containers.items():
                if node_name == info["name"]:
                    image = image_key
        if not image:
            selected_images = []
            for image, info in self._docker_containers.items():
                if info["server"] == node.server().host() or (
                        node.server().isLocal() and info["server"] == "local"):
                    selected_images.append(image)

            if not selected_images:
                raise ModuleError("No Docker VM on server {}".format(
                    node.server().url()))
            elif len(selected_images) > 1:
                from gns3.main_window import MainWindow
                mainwindow = MainWindow.instance()

                (selection, ok) = QtWidgets.QInputDialog.getItem(
                    mainwindow, "Docker Image", "Please choose an image",
                    selected_images, 0, False)
                if ok:
                    image = selection
                else:
                    raise ModuleError("Please select a Docker Image")
            else:
                image = selected_images[0]

        image_settings = {}
        for setting_name, value in self._docker_containers[image].items():
            if setting_name in node.settings() and value != "" and value is not None:
                if setting_name not in ['name', 'image']:
                    image_settings[setting_name] = value

        default_name_format = DOCKER_CONTAINER_SETTINGS["default_name_format"]
        if self._docker_containers[image]["default_name_format"]:
            default_name_format = self._docker_containers[image]["default_name_format"]

        image = self._docker_containers[image]["image"]
        node.setup(image, base_name=node_name, additional_settings=image_settings, default_name_format=default_name_format)

    def reset(self):
        """Resets the servers."""
        log.info("Docker module reset")
        self._nodes.clear()

    def getDockerImagesFromServer(self, server, callback):
        """Gets the Docker images list from a server.

        :param server: server to send the request to
        :param callback: callback for the reply from the server
        """
        server.get("/docker/images", callback)

    @staticmethod
    def getNodeClass(name):
        """
        Returns the object with the corresponding name.

        :param name: object name
        """
        if name in globals():
            return globals()[name]

    @staticmethod
    def classes():
        """Returns all the node classes supported by this module.

        :returns: list of classes
        """
        return [DockerVM]

    def nodes(self):
        """
        Returns all the node data necessary to represent a node
        in the nodes view and create a node on the scene.
        """
        nodes = []
        for docker_image in self._docker_containers.values():
            nodes.append({
                "class": DockerVM.__name__,
                "name": docker_image["name"],
                "server": docker_image["server"],
                "symbol": docker_image["symbol"],
                "categories": [docker_image["category"]]
            })
        return nodes

    @staticmethod
    def preferencePages():
        """
        :returns: QWidget object list
        """
        from .pages.docker_preferences_page import DockerPreferencesPage
        from .pages.docker_vm_preferences_page import DockerVMPreferencesPage

        return [DockerPreferencesPage, DockerVMPreferencesPage]

    @staticmethod
    def instance():
        """Singleton to return only one instance of Docker module.

        :returns: instance of Docker"""
        if not hasattr(Docker, "_instance"):
            Docker._instance = Docker()
        return Docker._instance



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from gns3.qt import QtWidgets
from gns3.ui.new_server_dialog_ui import Ui_NewServerDialog
from gns3.servers import Servers


class NewServerDialog(QtWidgets.QDialog, Ui_NewServerDialog):

    """
    New server dialog.

    :param parent: parent widget.
    has been opened automatically when GNS3 started.
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)
        self.uiEnableAuthenticationCheckBox.stateChanged.connect(self._enableAuthenticationSlot)

    def _enableAuthenticationSlot(self, state):
        """
        Slot to enable or not the authentication.
        """

        if state:
            self.uiServerUserLineEdit.setEnabled(True)
            self.uiServerPasswordLineEdit.setEnabled(True)
        else:
            self.uiServerUserLineEdit.setEnabled(False)
            self.uiServerPasswordLineEdit.setEnabled(False)

    def accept(self):
        """
        Adds a new remote server.
        """

        protocol = self.uiServerProtocolComboBox.currentText().lower()
        host = self.uiServerHostLineEdit.text().strip()
        port = self.uiServerPortSpinBox.value()
        if self.uiEnableAuthenticationCheckBox.isChecked():
            user = self.uiServerUserLineEdit.text().strip()
            password = self.uiServerPasswordLineEdit.text().strip()
        else:
            user = password = ""

        if not re.match(r"^[a-zA-Z0-9\.{}-]+$".format("\u0370-\u1CDF\u2C00-\u30FF\u4E00-\u9FBF"), host):
            QtWidgets.QMessageBox.critical(self, "Remote server", "Invalid remote server hostname {}".format(host))
            return
        if port is None or port < 1:
            QtWidgets.QMessageBox.critical(self, "Remote server", "Invalid remote server port {}".format(port))
            return

        servers = Servers.instance()
        remote_servers = servers.remoteServers()

        # check if the remote server is already defined
        for server in remote_servers.values():
            if server.protocol() == protocol and server.host() == host and server.port() == port and server.user() == user:
                QtWidgets.QMessageBox.critical(self, "Remote server", "Remote server is already defined.")
                return

        servers.getRemoteServer(protocol, host, port, user, settings={"password": password})
        servers.save()
        super().accept()


if __name__ == '__main__':
    import sys
    app = QtWidgets.QApplication(sys.argv)
    main = QtWidgets.QMainWindow()
    dialog = NewServerDialog(main)
    dialog.show()
    exit_code = app.exec_()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/docker/ui/docker_preferences_page.ui'
#
# Created: Thu May  5 18:51:18 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DockerPreferencesPageWidget(object):
    def setupUi(self, DockerPreferencesPageWidget):
        DockerPreferencesPageWidget.setObjectName("DockerPreferencesPageWidget")
        DockerPreferencesPageWidget.resize(330, 200)
        self.verticalLayout = QtWidgets.QVBoxLayout(DockerPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(DockerPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiServerSettingsTabWidget = QtWidgets.QWidget()
        self.uiServerSettingsTabWidget.setObjectName("uiServerSettingsTabWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiServerSettingsTabWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiUseLocalServercheckBox = QtWidgets.QCheckBox(self.uiServerSettingsTabWidget)
        self.uiUseLocalServercheckBox.setChecked(True)
        self.uiUseLocalServercheckBox.setObjectName("uiUseLocalServercheckBox")
        self.verticalLayout_2.addWidget(self.uiUseLocalServercheckBox)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiServerSettingsTabWidget, "")
        self.verticalLayout.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(254, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(DockerPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(DockerPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DockerPreferencesPageWidget)

    def retranslateUi(self, DockerPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        DockerPreferencesPageWidget.setWindowTitle(_translate("DockerPreferencesPageWidget", "Docker"))
        self.uiUseLocalServercheckBox.setText(_translate("DockerPreferencesPageWidget", "Use the local server"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiServerSettingsTabWidget), _translate("DockerPreferencesPageWidget", "General settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("DockerPreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from unittest.mock import patch, Mock
from gns3.modules.qemu.qemu_vm import QemuVM
from gns3.ports.port import Port
from gns3.nios.nio_udp import NIOUDP
from gns3.node import Node


def test_qemu_vm_init(local_server, project):

    vm = QemuVM(None, local_server, project)


def test_qemu_vm_setup(qemu_vm, project):

    with patch('gns3.node.Node.httpPost') as mock:
        qemu_vm.setup("/bin/fake", name="VMNAME")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/qemu/vms".format(project_id=project.id())

        # Callback
        params = {
            "name": "QEMU1",
            "vmname": "VMNAME",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "vm_directory": "/tmp/test",
            "hda_disk_image": "0cc175b9c0f1b6a831c399e269772661"
        }
        args[1](params)
        assert qemu_vm.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"
        assert qemu_vm.vmDir() == "/tmp/test"


def test_qemu_vm_setup_command_line(qemu_vm, project):

    with patch('gns3.node.Node.httpPost') as mock:
        qemu_vm.setup("/bin/fake", name="VMNAME")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/qemu/vms".format(project_id=project.id())

        # Callback
        params = {
            "name": "QEMU1",
            "vmname": "VMNAME",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "vm_directory": "/tmp/test",
            "hda_disk_image": "0cc175b9c0f1b6a831c399e269772661",
            "command_line": "/bin/fake"
        }
        args[1](params)
        assert qemu_vm.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"
        assert qemu_vm.commandLine() == "/bin/fake"


def test_qemu_vm_setup_md5_missing(qemu_vm, project):

    with patch('gns3.node.Node.httpPost') as mock:
        qemu_vm.setup("/bin/fake", name="VMNAME")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/qemu/vms".format(project_id=project.id())

        # Callback
        params = {
            "name": "QEMU1",
            "vmname": "VMNAME",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "hda_disk_image": "0cc175b9c0f1b6a831c399e269772661"
        }
        with patch("gns3.image_manager.ImageManager.addMissingImage") as mock:
            args[1](params)
            assert mock.called

        assert qemu_vm.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"


def test_update(qemu_vm):

    new_settings = {
        "name": "QEMU2",
    }

    with patch('gns3.node.Node.httpPut') as mock:
        qemu_vm.update(new_settings)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/qemu/vms/{vm_id}".format(vm_id=qemu_vm.vm_id())
        assert kwargs["body"] == new_settings

        # Callback
        args[1]({"name": "QEMU2"})
